@@ -1,6 +1,9 @@
-from helper import unittest, PillowTestCase
+import unittest
+
 from PIL import Image
 
+from .helper import PillowTestCase
+
 TEST_FILE = "Tests/images/fli_overflow.fli"
 
 
@@ -8,9 +11,9 @@ class TestFliOverflow(PillowTestCase):
     def test_fli_overflow(self):
 
         # this should not crash with a malloc error or access violation
-        im = Image.open(TEST_FILE)
-        im.load()
+        with Image.open(TEST_FILE) as im:
+            im.load()
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
