@@ -1,6 +1,7 @@
-from helper import unittest, PillowTestCase, hopper
+import unittest
 from io import BytesIO
-import sys
+
+from .helper import PillowTestCase, hopper, is_win32
 
 iterations = 5000
 
@@ -14,7 +15,7 @@ valgrind --tool=massif python test-installed.py -s -v Tests/check_jpeg_leaks.py
 """
 
 
-@unittest.skipIf(sys.platform.startswith('win32'), "requires Unix or macOS")
+@unittest.skipIf(is_win32(), "requires Unix or macOS")
 class TestJpegLeaks(PillowTestCase):
 
     """
@@ -74,9 +75,11 @@ post-patch:
 """
 
     def test_qtables_leak(self):
-        im = hopper('RGB')
+        im = hopper("RGB")
 
-        standard_l_qtable = [int(s) for s in """
+        standard_l_qtable = [
+            int(s)
+            for s in """
             16  11  10  16  24  40  51  61
             12  12  14  19  26  58  60  55
             14  13  16  24  40  57  69  56
@@ -85,9 +88,14 @@ post-patch:
             24  35  55  64  81 104 113  92
             49  64  78  87 103 121 120 101
             72  92  95  98 112 100 103  99
-            """.split(None)]
+            """.split(
+                None
+            )
+        ]
 
-        standard_chrominance_qtable = [int(s) for s in """
+        standard_chrominance_qtable = [
+            int(s)
+            for s in """
             17  18  24  47  99  99  99  99
             18  21  26  66  99  99  99  99
             24  26  56  99  99  99  99  99
@@ -96,10 +104,12 @@ post-patch:
             99  99  99  99  99  99  99  99
             99  99  99  99  99  99  99  99
             99  99  99  99  99  99  99  99
-            """.split(None)]
+            """.split(
+                None
+            )
+        ]
 
-        qtables = [standard_l_qtable,
-                   standard_chrominance_qtable]
+        qtables = [standard_l_qtable, standard_chrominance_qtable]
 
         for _ in range(iterations):
             test_output = BytesIO()
@@ -161,8 +171,8 @@ post patch:
      0                                                                   11.33
 
 """
-        im = hopper('RGB')
-        exif = b'12345678'*4096
+        im = hopper("RGB")
+        exif = b"12345678" * 4096
 
         for _ in range(iterations):
             test_output = BytesIO()
@@ -195,12 +205,12 @@ base case:
    0 +----------------------------------------------------------------------->Gi
      0                                                                   7.882
 """
-        im = hopper('RGB')
+        im = hopper("RGB")
 
         for _ in range(iterations):
             test_output = BytesIO()
             im.save(test_output, "JPEG")
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
