@@ -1,6 +1,9 @@
 import sys
+import unittest
 
-from .helper import unittest, PillowTestCase
+from PIL import Image
+
+from .helper import PillowTestCase
 
 # This test is not run automatically.
 #
@@ -10,7 +13,6 @@ from .helper import unittest, PillowTestCase
 # on any 32-bit machine, as well as any smallish things (like
 # Raspberry Pis).
 
-from PIL import Image
 
 try:
     import numpy as np
