@@ -1,9 +1,9 @@
-from __future__ import division
-
+import unittest
 from array import array
 
 from PIL import Image, ImageFilter
-from .helper import unittest, PillowTestCase
+
+from .helper import PillowTestCase
 
 try:
     import numpy
