@@ -1,4 +1,8 @@
-from .helper import unittest, PillowTestCase
+import unittest
+
+from PIL import Image
+
+from .helper import PillowTestCase
 
 try:
     from PIL import FpxImagePlugin
@@ -18,3 +22,7 @@ class TestFileFpx(PillowTestCase):
         # Test a valid OLE file, but not an FPX file
         ole_file = "Tests/images/test-ole-file.doc"
         self.assertRaises(SyntaxError, FpxImagePlugin.FpxImageFile, ole_file)
+
+    def test_fpx_invalid_number_of_bands(self):
+        with self.assertRaisesRegex(IOError, "Invalid number of bands"):
+            Image.open("Tests/images/input_bw_five_bands.fpx")
