@@ -1,28 +1,30 @@
-from helper import unittest, PillowTestCase, hopper
+import sys
+from io import StringIO
 
 from PIL import Image, IptcImagePlugin
 
+from .helper import PillowTestCase, hopper
+
 TEST_FILE = "Tests/images/iptc.jpg"
 
 
 class TestFileIptc(PillowTestCase):
-
     def test_getiptcinfo_jpg_none(self):
         # Arrange
-        im = hopper()
+        with hopper() as im:
 
-        # Act
-        iptc = IptcImagePlugin.getiptcinfo(im)
+            # Act
+            iptc = IptcImagePlugin.getiptcinfo(im)
 
         # Assert
         self.assertIsNone(iptc)
 
     def test_getiptcinfo_jpg_found(self):
         # Arrange
-        im = Image.open(TEST_FILE)
+        with Image.open(TEST_FILE) as im:
 
-        # Act
-        iptc = IptcImagePlugin.getiptcinfo(im)
+            # Act
+            iptc = IptcImagePlugin.getiptcinfo(im)
 
         # Assert
         self.assertIsInstance(iptc, dict)
@@ -31,10 +33,10 @@ class TestFileIptc(PillowTestCase):
 
     def test_getiptcinfo_tiff_none(self):
         # Arrange
-        im = Image.open("Tests/images/hopper.tif")
+        with Image.open("Tests/images/hopper.tif") as im:
 
-        # Act
-        iptc = IptcImagePlugin.getiptcinfo(im)
+            # Act
+            iptc = IptcImagePlugin.getiptcinfo(im)
 
         # Assert
         self.assertIsNone(iptc)
@@ -53,11 +55,6 @@ class TestFileIptc(PillowTestCase):
         # Arrange
         c = b"abc"
         # Temporarily redirect stdout
-        try:
-            from cStringIO import StringIO
-        except ImportError:
-            from io import StringIO
-        import sys
         old_stdout = sys.stdout
         sys.stdout = mystdout = StringIO()
 
@@ -69,7 +66,3 @@ class TestFileIptc(PillowTestCase):
 
         # Assert
         self.assertEqual(mystdout.getvalue(), "61 62 63 \n")
-
-
-if __name__ == '__main__':
-    unittest.main()
