@@ -1,15 +1,13 @@
-from helper import unittest, PillowTestCase
-
 from PIL import Image, McIdasImagePlugin
 
+from .helper import PillowTestCase
+
 
 class TestFileMcIdas(PillowTestCase):
-
     def test_invalid_file(self):
         invalid_file = "Tests/images/flower.jpg"
 
-        self.assertRaises(SyntaxError,
-                          McIdasImagePlugin.McIdasImageFile, invalid_file)
+        self.assertRaises(SyntaxError, McIdasImagePlugin.McIdasImageFile, invalid_file)
 
     def test_valid_file(self):
         # Arrange
@@ -19,16 +17,12 @@ class TestFileMcIdas(PillowTestCase):
         saved_file = "Tests/images/cmx3g8_wv_1998.260_0745_mcidas.png"
 
         # Act
-        im = Image.open(test_file)
-        im.load()
+        with Image.open(test_file) as im:
+            im.load()
 
-        # Assert
-        self.assertEqual(im.format, "MCIDAS")
-        self.assertEqual(im.mode, "I")
-        self.assertEqual(im.size, (1800, 400))
-        im2 = Image.open(saved_file)
-        self.assert_image_equal(im, im2)
-
-
-if __name__ == '__main__':
-    unittest.main()
+            # Assert
+            self.assertEqual(im.format, "MCIDAS")
+            self.assertEqual(im.mode, "I")
+            self.assertEqual(im.size, (1800, 400))
+            with Image.open(saved_file) as im2:
+                self.assert_image_equal(im, im2)
