@@ -1,7 +1,7 @@
-from helper import unittest, PillowTestCase, hopper, imagemagick_available
-
 import os.path
 
+from .helper import PillowTestCase, hopper, imagemagick_available
+
 
 class TestFilePalm(PillowTestCase):
     _roundtrip = imagemagick_available()
@@ -46,13 +46,16 @@ class TestFilePalm(PillowTestCase):
         self.skipKnownBadTest("Palm P image is wrong")
         self.roundtrip(mode)
 
+    def test_l_ioerror(self):
+        # Arrange
+        mode = "L"
+
+        # Act / Assert
+        self.assertRaises(IOError, self.helper_save_as_palm, mode)
+
     def test_rgb_ioerror(self):
         # Arrange
         mode = "RGB"
 
         # Act / Assert
         self.assertRaises(IOError, self.helper_save_as_palm, mode)
-
-
-if __name__ == '__main__':
-    unittest.main()
