@@ -1,27 +1,43 @@
-from .helper import PillowTestCase, hopper
-
-from PIL import Image
-from PIL import ImageSequence
-from PIL import SpiderImagePlugin
-
 import tempfile
+import unittest
+from io import BytesIO
+
+from PIL import Image, ImageSequence, SpiderImagePlugin
+
+from .helper import PillowTestCase, hopper, is_pypy
 
 TEST_FILE = "Tests/images/hopper.spider"
 
 
 class TestImageSpider(PillowTestCase):
     def test_sanity(self):
-        im = Image.open(TEST_FILE)
-        im.load()
-        self.assertEqual(im.mode, "F")
-        self.assertEqual(im.size, (128, 128))
-        self.assertEqual(im.format, "SPIDER")
+        with Image.open(TEST_FILE) as im:
+            im.load()
+            self.assertEqual(im.mode, "F")
+            self.assertEqual(im.size, (128, 128))
+            self.assertEqual(im.format, "SPIDER")
 
+    @unittest.skipIf(is_pypy(), "Requires CPython")
     def test_unclosed_file(self):
         def open():
             im = Image.open(TEST_FILE)
             im.load()
 
+        self.assert_warning(ResourceWarning, open)
+
+    def test_closed_file(self):
+        def open():
+            im = Image.open(TEST_FILE)
+            im.load()
+            im.close()
+
+        self.assert_warning(None, open)
+
+    def test_context_manager(self):
+        def open():
+            with Image.open(TEST_FILE) as im:
+                im.load()
+
         self.assert_warning(None, open)
 
     def test_save(self):
@@ -33,10 +49,10 @@ class TestImageSpider(PillowTestCase):
         im.save(temp, "SPIDER")
 
         # Assert
-        im2 = Image.open(temp)
-        self.assertEqual(im2.mode, "F")
-        self.assertEqual(im2.size, (128, 128))
-        self.assertEqual(im2.format, "SPIDER")
+        with Image.open(temp) as im2:
+            self.assertEqual(im2.mode, "F")
+            self.assertEqual(im2.size, (128, 128))
+            self.assertEqual(im2.format, "SPIDER")
 
     def test_tempfile(self):
         # Arrange
@@ -48,28 +64,28 @@ class TestImageSpider(PillowTestCase):
 
             # Assert
             fp.seek(0)
-            reloaded = Image.open(fp)
-            self.assertEqual(reloaded.mode, "F")
-            self.assertEqual(reloaded.size, (128, 128))
-            self.assertEqual(reloaded.format, "SPIDER")
+            with Image.open(fp) as reloaded:
+                self.assertEqual(reloaded.mode, "F")
+                self.assertEqual(reloaded.size, (128, 128))
+                self.assertEqual(reloaded.format, "SPIDER")
 
     def test_isSpiderImage(self):
         self.assertTrue(SpiderImagePlugin.isSpiderImage(TEST_FILE))
 
     def test_tell(self):
         # Arrange
-        im = Image.open(TEST_FILE)
+        with Image.open(TEST_FILE) as im:
 
-        # Act
-        index = im.tell()
+            # Act
+            index = im.tell()
 
-        # Assert
-        self.assertEqual(index, 0)
+            # Assert
+            self.assertEqual(index, 0)
 
     def test_n_frames(self):
-        im = Image.open(TEST_FILE)
-        self.assertEqual(im.n_frames, 1)
-        self.assertFalse(im.is_animated)
+        with Image.open(TEST_FILE) as im:
+            self.assertEqual(im.n_frames, 1)
+            self.assertFalse(im.is_animated)
 
     def test_loadImageSeries(self):
         # Arrange
@@ -110,12 +126,22 @@ class TestImageSpider(PillowTestCase):
         self.assertRaises(IOError, Image.open, invalid_file)
 
     def test_nonstack_file(self):
-        im = Image.open(TEST_FILE)
-
-        self.assertRaises(EOFError, im.seek, 0)
+        with Image.open(TEST_FILE) as im:
+            self.assertRaises(EOFError, im.seek, 0)
 
     def test_nonstack_dos(self):
-        im = Image.open(TEST_FILE)
-        for i, frame in enumerate(ImageSequence.Iterator(im)):
-            if i > 1:
-                self.fail("Non-stack DOS file test failed")
+        with Image.open(TEST_FILE) as im:
+            for i, frame in enumerate(ImageSequence.Iterator(im)):
+                if i > 1:
+                    self.fail("Non-stack DOS file test failed")
+
+    # for issue #4093
+    def test_odd_size(self):
+        data = BytesIO()
+        width = 100
+        im = Image.new("F", (width, 64))
+        im.save(data, format="SPIDER")
+
+        data.seek(0)
+        with Image.open(data) as im2:
+            self.assert_image_equal(im, im2)
