@@ -1,8 +1,6 @@
-from .helper import PillowTestCase
+from PIL import FontFile, Image, ImageDraw, ImageFont, PcfFontFile
 
-from PIL import Image, FontFile, PcfFontFile
-from PIL import ImageFont, ImageDraw
-from PIL._util import py3
+from .helper import PillowTestCase
 
 codecs = dir(Image.core)
 
@@ -75,6 +73,5 @@ class TestFontPcf(PillowTestCase):
     def test_high_characters(self):
         message = "".join(chr(i + 1) for i in range(140, 232))
         self._test_high_characters(message)
-        # accept bytes instances in Py3.
-        if py3:
-            self._test_high_characters(message.encode("latin1"))
+        # accept bytes instances.
+        self._test_high_characters(message.encode("latin1"))
