@@ -1,14 +1,14 @@
-from helper import unittest, PillowTestCase, hopper
+from .helper import PillowTestCase, hopper
 
 
 class TestImageGetPalette(PillowTestCase):
-
     def test_palette(self):
         def palette(mode):
             p = hopper(mode).getpalette()
             if p:
                 return p[:10]
             return None
+
         self.assertIsNone(palette("1"))
         self.assertIsNone(palette("L"))
         self.assertIsNone(palette("I"))
@@ -18,7 +18,3 @@ class TestImageGetPalette(PillowTestCase):
         self.assertIsNone(palette("RGBA"))
         self.assertIsNone(palette("CMYK"))
         self.assertIsNone(palette("YCbCr"))
-
-
-if __name__ == '__main__':
-    unittest.main()
