@@ -1,20 +1,24 @@
-from __future__ import division, print_function
-
+import unittest
 from contextlib import contextmanager
 
-from helper import unittest, PillowTestCase, hopper
 from PIL import Image, ImageDraw
 
+from .helper import PillowTestCase, hopper
+
 
 class TestImagingResampleVulnerability(PillowTestCase):
     # see https://github.com/python-pillow/Pillow/issues/1710
     def test_overflow(self):
-        im = hopper('L')
-        xsize = 0x100000008 // 4
-        ysize = 1000  # unimportant
-        with self.assertRaises(MemoryError):
-            # any resampling filter will do here
-            im.im.resize((xsize, ysize), Image.BILINEAR)
+        im = hopper("L")
+        size_too_large = 0x100000008 // 4
+        size_normal = 1000  # unimportant
+        for xsize, ysize in (
+            (size_too_large, size_normal),
+            (size_normal, size_too_large),
+        ):
+            with self.assertRaises(MemoryError):
+                # any resampling filter will do here
+                im.im.resize((xsize, ysize), Image.BILINEAR)
 
     def test_invalid_size(self):
         im = hopper()
@@ -29,11 +33,11 @@ class TestImagingResampleVulnerability(PillowTestCase):
             im.resize((100, -100))
 
     def test_modify_after_resizing(self):
-        im = hopper('RGB')
+        im = hopper("RGB")
         # get copy with same size
         copy = im.resize(im.size)
         # some in-place operation
-        copy.paste('black', (0, 0, im.width // 2, im.height // 2))
+        copy.paste("black", (0, 0, im.width // 2, im.height // 2))
         # image should be different
         self.assertNotEqual(im.tobytes(), copy.tobytes())
 
@@ -47,7 +51,7 @@ class TestImagingCoreResampleAccuracy(PillowTestCase):
         1f 1f e0 e0
         1f 1f e0 e0
         """
-        case = Image.new('L', size, 255 - color)
+        case = Image.new("L", size, 255 - color)
         rectangle = ImageDraw.Draw(case).rectangle
         rectangle((0, 0, size[0] // 2 - 1, size[1] // 2 - 1), color)
         rectangle((size[0] // 2, size[1] // 2, size[0], size[1]), color)
@@ -58,13 +62,13 @@ class TestImagingCoreResampleAccuracy(PillowTestCase):
         """Restores a sample image from given data string which contains
         hex-encoded pixels from the top left fourth of a sample.
         """
-        data = data.replace(' ', '')
-        sample = Image.new('L', size)
+        data = data.replace(" ", "")
+        sample = Image.new("L", size)
         s_px = sample.load()
         w, h = size[0] // 2, size[1] // 2
         for y in range(h):
             for x in range(w):
-                val = int(data[(y * w + x) * 2:(y * w + x + 1) * 2], 16)
+                val = int(data[(y * w + x) * 2 : (y * w + x + 1) * 2], 16)
                 s_px[x, y] = val
                 s_px[size[0] - x - 1, size[1] - y - 1] = val
                 s_px[x, size[1] - y - 1] = 255 - val
@@ -77,121 +81,142 @@ class TestImagingCoreResampleAccuracy(PillowTestCase):
         for y in range(case.size[1]):
             for x in range(case.size[0]):
                 if c_px[x, y] != s_px[x, y]:
-                    message = '\nHave: \n{}\n\nExpected: \n{}'.format(
-                        self.serialize_image(case),
-                        self.serialize_image(sample),
+                    message = "\nHave: \n{}\n\nExpected: \n{}".format(
+                        self.serialize_image(case), self.serialize_image(sample)
                     )
                     self.assertEqual(s_px[x, y], c_px[x, y], message)
 
     def serialize_image(self, image):
         s_px = image.load()
-        return '\n'.join(
-            ' '.join(
-                '{:02x}'.format(s_px[x, y])
-                for x in range(image.size[0])
-            )
+        return "\n".join(
+            " ".join("{:02x}".format(s_px[x, y]) for x in range(image.size[0]))
             for y in range(image.size[1])
         )
 
     def test_reduce_box(self):
-        for mode in ['RGBX', 'RGB', 'La', 'L']:
-            case = self.make_case(mode, (8, 8), 0xe1)
+        for mode in ["RGBX", "RGB", "La", "L"]:
+            case = self.make_case(mode, (8, 8), 0xE1)
             case = case.resize((4, 4), Image.BOX)
-            data = ('e1 e1'
-                    'e1 e1')
+            # fmt: off
+            data = ("e1 e1"
+                    "e1 e1")
+            # fmt: on
             for channel in case.split():
                 self.check_case(channel, self.make_sample(data, (4, 4)))
 
     def test_reduce_bilinear(self):
-        for mode in ['RGBX', 'RGB', 'La', 'L']:
-            case = self.make_case(mode, (8, 8), 0xe1)
+        for mode in ["RGBX", "RGB", "La", "L"]:
+            case = self.make_case(mode, (8, 8), 0xE1)
             case = case.resize((4, 4), Image.BILINEAR)
-            data = ('e1 c9'
-                    'c9 b7')
+            # fmt: off
+            data = ("e1 c9"
+                    "c9 b7")
+            # fmt: on
             for channel in case.split():
                 self.check_case(channel, self.make_sample(data, (4, 4)))
 
     def test_reduce_hamming(self):
-        for mode in ['RGBX', 'RGB', 'La', 'L']:
-            case = self.make_case(mode, (8, 8), 0xe1)
+        for mode in ["RGBX", "RGB", "La", "L"]:
+            case = self.make_case(mode, (8, 8), 0xE1)
             case = case.resize((4, 4), Image.HAMMING)
-            data = ('e1 da'
-                    'da d3')
+            # fmt: off
+            data = ("e1 da"
+                    "da d3")
+            # fmt: on
             for channel in case.split():
                 self.check_case(channel, self.make_sample(data, (4, 4)))
 
     def test_reduce_bicubic(self):
-        for mode in ['RGBX', 'RGB', 'La', 'L']:
-            case = self.make_case(mode, (12, 12), 0xe1)
+        for mode in ["RGBX", "RGB", "La", "L"]:
+            case = self.make_case(mode, (12, 12), 0xE1)
             case = case.resize((6, 6), Image.BICUBIC)
-            data = ('e1 e3 d4'
-                    'e3 e5 d6'
-                    'd4 d6 c9')
+            # fmt: off
+            data = ("e1 e3 d4"
+                    "e3 e5 d6"
+                    "d4 d6 c9")
+            # fmt: on
             for channel in case.split():
                 self.check_case(channel, self.make_sample(data, (6, 6)))
 
     def test_reduce_lanczos(self):
-        for mode in ['RGBX', 'RGB', 'La', 'L']:
-            case = self.make_case(mode, (16, 16), 0xe1)
+        for mode in ["RGBX", "RGB", "La", "L"]:
+            case = self.make_case(mode, (16, 16), 0xE1)
             case = case.resize((8, 8), Image.LANCZOS)
-            data = ('e1 e0 e4 d7'
-                    'e0 df e3 d6'
-                    'e4 e3 e7 da'
-                    'd7 d6 d9 ce')
+            # fmt: off
+            data = ("e1 e0 e4 d7"
+                    "e0 df e3 d6"
+                    "e4 e3 e7 da"
+                    "d7 d6 d9 ce")
+            # fmt: on
             for channel in case.split():
                 self.check_case(channel, self.make_sample(data, (8, 8)))
 
     def test_enlarge_box(self):
-        for mode in ['RGBX', 'RGB', 'La', 'L']:
-            case = self.make_case(mode, (2, 2), 0xe1)
+        for mode in ["RGBX", "RGB", "La", "L"]:
+            case = self.make_case(mode, (2, 2), 0xE1)
             case = case.resize((4, 4), Image.BOX)
-            data = ('e1 e1'
-                    'e1 e1')
+            # fmt: off
+            data = ("e1 e1"
+                    "e1 e1")
+            # fmt: on
             for channel in case.split():
                 self.check_case(channel, self.make_sample(data, (4, 4)))
 
     def test_enlarge_bilinear(self):
-        for mode in ['RGBX', 'RGB', 'La', 'L']:
-            case = self.make_case(mode, (2, 2), 0xe1)
+        for mode in ["RGBX", "RGB", "La", "L"]:
+            case = self.make_case(mode, (2, 2), 0xE1)
             case = case.resize((4, 4), Image.BILINEAR)
-            data = ('e1 b0'
-                    'b0 98')
+            # fmt: off
+            data = ("e1 b0"
+                    "b0 98")
+            # fmt: on
             for channel in case.split():
                 self.check_case(channel, self.make_sample(data, (4, 4)))
 
     def test_enlarge_hamming(self):
-        for mode in ['RGBX', 'RGB', 'La', 'L']:
-            case = self.make_case(mode, (2, 2), 0xe1)
+        for mode in ["RGBX", "RGB", "La", "L"]:
+            case = self.make_case(mode, (2, 2), 0xE1)
             case = case.resize((4, 4), Image.HAMMING)
-            data = ('e1 d2'
-                    'd2 c5')
+            # fmt: off
+            data = ("e1 d2"
+                    "d2 c5")
+            # fmt: on
             for channel in case.split():
                 self.check_case(channel, self.make_sample(data, (4, 4)))
 
     def test_enlarge_bicubic(self):
-        for mode in ['RGBX', 'RGB', 'La', 'L']:
-            case = self.make_case(mode, (4, 4), 0xe1)
+        for mode in ["RGBX", "RGB", "La", "L"]:
+            case = self.make_case(mode, (4, 4), 0xE1)
             case = case.resize((8, 8), Image.BICUBIC)
-            data = ('e1 e5 ee b9'
-                    'e5 e9 f3 bc'
-                    'ee f3 fd c1'
-                    'b9 bc c1 a2')
+            # fmt: off
+            data = ("e1 e5 ee b9"
+                    "e5 e9 f3 bc"
+                    "ee f3 fd c1"
+                    "b9 bc c1 a2")
+            # fmt: on
             for channel in case.split():
                 self.check_case(channel, self.make_sample(data, (8, 8)))
 
     def test_enlarge_lanczos(self):
-        for mode in ['RGBX', 'RGB', 'La', 'L']:
-            case = self.make_case(mode, (6, 6), 0xe1)
+        for mode in ["RGBX", "RGB", "La", "L"]:
+            case = self.make_case(mode, (6, 6), 0xE1)
             case = case.resize((12, 12), Image.LANCZOS)
-            data = ('e1 e0 db ed f5 b8'
-                    'e0 df da ec f3 b7'
-                    'db db d6 e7 ee b5'
-                    'ed ec e6 fb ff bf'
-                    'f5 f4 ee ff ff c4'
-                    'b8 b7 b4 bf c4 a0')
+            data = (
+                "e1 e0 db ed f5 b8"
+                "e0 df da ec f3 b7"
+                "db db d6 e7 ee b5"
+                "ed ec e6 fb ff bf"
+                "f5 f4 ee ff ff c4"
+                "b8 b7 b4 bf c4 a0"
+            )
             for channel in case.split():
                 self.check_case(channel, self.make_sample(data, (12, 12)))
 
+    def test_box_filter_correct_range(self):
+        im = Image.new("RGB", (8, 8), "#1688ff").resize((100, 100), Image.BOX)
+        ref = Image.new("RGB", (100, 100), "#1688ff")
+        self.assert_image_equal(im, ref)
+
 
 class CoreResampleConsistencyTest(PillowTestCase):
     def make_case(self, mode, fill):
@@ -204,29 +229,28 @@ class CoreResampleConsistencyTest(PillowTestCase):
         for x in range(channel.size[0]):
             for y in range(channel.size[1]):
                 if px[x, y] != color:
-                    message = "{} != {} for pixel {}".format(
-                        px[x, y], color, (x, y))
+                    message = "{} != {} for pixel {}".format(px[x, y], color, (x, y))
                     self.assertEqual(px[x, y], color, message)
 
     def test_8u(self):
-        im, color = self.make_case('RGB', (0, 64, 255))
+        im, color = self.make_case("RGB", (0, 64, 255))
         r, g, b = im.split()
         self.run_case((r, color[0]))
         self.run_case((g, color[1]))
         self.run_case((b, color[2]))
-        self.run_case(self.make_case('L', 12))
+        self.run_case(self.make_case("L", 12))
 
     def test_32i(self):
-        self.run_case(self.make_case('I', 12))
-        self.run_case(self.make_case('I', 0x7fffffff))
-        self.run_case(self.make_case('I', -12))
-        self.run_case(self.make_case('I', -1 << 31))
+        self.run_case(self.make_case("I", 12))
+        self.run_case(self.make_case("I", 0x7FFFFFFF))
+        self.run_case(self.make_case("I", -12))
+        self.run_case(self.make_case("I", -1 << 31))
 
     def test_32f(self):
-        self.run_case(self.make_case('F', 1))
-        self.run_case(self.make_case('F', 3.40282306074e+38))
-        self.run_case(self.make_case('F', 1.175494e-38))
-        self.run_case(self.make_case('F', 1.192093e-07))
+        self.run_case(self.make_case("F", 1))
+        self.run_case(self.make_case("F", 3.40282306074e38))
+        self.run_case(self.make_case("F", 1.175494e-38))
+        self.run_case(self.make_case("F", 1.192093e-07))
 
 
 class CoreResampleAlphaCorrectTest(PillowTestCase):
@@ -244,13 +268,16 @@ class CoreResampleAlphaCorrectTest(PillowTestCase):
         px = i.load()
         for y in range(i.size[1]):
             used_colors = {px[x, y][0] for x in range(i.size[0])}
-            self.assertEqual(256, len(used_colors),
-                             'All colors should present in resized image. '
-                             'Only {} on {} line.'.format(len(used_colors), y))
+            self.assertEqual(
+                256,
+                len(used_colors),
+                "All colors should present in resized image. "
+                "Only {} on {} line.".format(len(used_colors), y),
+            )
 
     @unittest.skip("current implementation isn't precise enough")
     def test_levels_rgba(self):
-        case = self.make_levels_case('RGBA')
+        case = self.make_levels_case("RGBA")
         self.run_levels_case(case.resize((512, 32), Image.BOX))
         self.run_levels_case(case.resize((512, 32), Image.BILINEAR))
         self.run_levels_case(case.resize((512, 32), Image.HAMMING))
@@ -259,7 +286,7 @@ class CoreResampleAlphaCorrectTest(PillowTestCase):
 
     @unittest.skip("current implementation isn't precise enough")
     def test_levels_la(self):
-        case = self.make_levels_case('LA')
+        case = self.make_levels_case("LA")
         self.run_levels_case(case.resize((512, 32), Image.BOX))
         self.run_levels_case(case.resize((512, 32), Image.BILINEAR))
         self.run_levels_case(case.resize((512, 32), Image.HAMMING))
@@ -281,24 +308,21 @@ class CoreResampleAlphaCorrectTest(PillowTestCase):
         for y in range(i.size[1]):
             for x in range(i.size[0]):
                 if px[x, y][-1] != 0 and px[x, y][:-1] != clean_pixel:
-                    message = 'pixel at ({}, {}) is differ:\n{}\n{}'\
-                        .format(x, y, px[x, y], clean_pixel)
+                    message = "pixel at ({}, {}) is differ:\n{}\n{}".format(
+                        x, y, px[x, y], clean_pixel
+                    )
                     self.assertEqual(px[x, y][:3], clean_pixel, message)
 
     def test_dirty_pixels_rgba(self):
-        case = self.make_dirty_case('RGBA', (255, 255, 0, 128), (0, 0, 255, 0))
+        case = self.make_dirty_case("RGBA", (255, 255, 0, 128), (0, 0, 255, 0))
         self.run_dirty_case(case.resize((20, 20), Image.BOX), (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.BILINEAR),
-                            (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.HAMMING),
-                            (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.BICUBIC),
-                            (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.LANCZOS),
-                            (255, 255, 0))
+        self.run_dirty_case(case.resize((20, 20), Image.BILINEAR), (255, 255, 0))
+        self.run_dirty_case(case.resize((20, 20), Image.HAMMING), (255, 255, 0))
+        self.run_dirty_case(case.resize((20, 20), Image.BICUBIC), (255, 255, 0))
+        self.run_dirty_case(case.resize((20, 20), Image.LANCZOS), (255, 255, 0))
 
     def test_dirty_pixels_la(self):
-        case = self.make_dirty_case('LA', (255, 128), (0, 0))
+        case = self.make_dirty_case("LA", (255, 128), (0, 0))
         self.run_dirty_case(case.resize((20, 20), Image.BOX), (255,))
         self.run_dirty_case(case.resize((20, 20), Image.BILINEAR), (255,))
         self.run_dirty_case(case.resize((20, 20), Image.HAMMING), (255,))
@@ -309,27 +333,27 @@ class CoreResampleAlphaCorrectTest(PillowTestCase):
 class CoreResamplePassesTest(PillowTestCase):
     @contextmanager
     def count(self, diff):
-        count = Image.core.get_stats()['new_count']
+        count = Image.core.get_stats()["new_count"]
         yield
-        self.assertEqual(Image.core.get_stats()['new_count'] - count, diff)
+        self.assertEqual(Image.core.get_stats()["new_count"] - count, diff)
 
     def test_horizontal(self):
-        im = hopper('L')
+        im = hopper("L")
         with self.count(1):
             im.resize((im.size[0] - 10, im.size[1]), Image.BILINEAR)
 
     def test_vertical(self):
-        im = hopper('L')
+        im = hopper("L")
         with self.count(1):
             im.resize((im.size[0], im.size[1] - 10), Image.BILINEAR)
 
     def test_both(self):
-        im = hopper('L')
+        im = hopper("L")
         with self.count(2):
             im.resize((im.size[0] - 10, im.size[1] - 10), Image.BILINEAR)
 
     def test_box_horizontal(self):
-        im = hopper('L')
+        im = hopper("L")
         box = (20, 0, im.size[0] - 20, im.size[1])
         with self.count(1):
             # the same size, but different box
@@ -339,7 +363,7 @@ class CoreResamplePassesTest(PillowTestCase):
         self.assert_image_similar(with_box, cropped, 0.1)
 
     def test_box_vertical(self):
-        im = hopper('L')
+        im = hopper("L")
         box = (0, 20, im.size[0], im.size[1] - 20)
         with self.count(1):
             # the same size, but different box
@@ -354,7 +378,7 @@ class CoreResampleCoefficientsTest(PillowTestCase):
         test_color = 254
 
         for size in range(400000, 400010, 2):
-            i = Image.new('L', (size, 1), 0)
+            i = Image.new("L", (size, 1), 0)
             draw = ImageDraw.Draw(i)
             draw.rectangle((0, 0, i.size[0] // 2 - 1, 0), test_color)
 
@@ -365,7 +389,7 @@ class CoreResampleCoefficientsTest(PillowTestCase):
     def test_nonzero_coefficients(self):
         # regression test for the wrong coefficients calculation
         # due to bug https://github.com/python-pillow/Pillow/issues/2161
-        im = Image.new('RGBA', (1280, 1280), (0x20, 0x40, 0x60, 0xff))
+        im = Image.new("RGBA", (1280, 1280), (0x20, 0x40, 0x60, 0xFF))
         histogram = im.resize((256, 256), Image.BICUBIC).histogram()
 
         # first channel
@@ -375,21 +399,26 @@ class CoreResampleCoefficientsTest(PillowTestCase):
         # third channel
         self.assertEqual(histogram[0x100 * 2 + 0x60], 0x10000)
         # fourth channel
-        self.assertEqual(histogram[0x100 * 3 + 0xff], 0x10000)
+        self.assertEqual(histogram[0x100 * 3 + 0xFF], 0x10000)
 
 
 class CoreResampleBoxTest(PillowTestCase):
     def test_wrong_arguments(self):
         im = hopper()
-        for resample in (Image.NEAREST, Image.BOX, Image.BILINEAR,
-                         Image.HAMMING, Image.BICUBIC, Image.LANCZOS):
+        for resample in (
+            Image.NEAREST,
+            Image.BOX,
+            Image.BILINEAR,
+            Image.HAMMING,
+            Image.BICUBIC,
+            Image.LANCZOS,
+        ):
             im.resize((32, 32), resample, (0, 0, im.width, im.height))
             im.resize((32, 32), resample, (20, 20, im.width, im.height))
             im.resize((32, 32), resample, (20, 20, 20, 100))
             im.resize((32, 32), resample, (20, 20, 100, 20))
 
-            with self.assertRaisesRegex(TypeError,
-                                        "must be sequence of length 4"):
+            with self.assertRaisesRegex(TypeError, "must be sequence of length 4"):
                 im.resize((32, 32), resample, (im.width, im.height))
 
             with self.assertRaisesRegex(ValueError, "can't be negative"):
@@ -420,35 +449,33 @@ class CoreResampleBoxTest(PillowTestCase):
 
         for y0, y1 in split_range(dst_size[1], ytiles):
             for x0, x1 in split_range(dst_size[0], xtiles):
-                box = (x0 * scale[0], y0 * scale[1],
-                       x1 * scale[0], y1 * scale[1])
+                box = (x0 * scale[0], y0 * scale[1], x1 * scale[0], y1 * scale[1])
                 tile = im.resize((x1 - x0, y1 - y0), Image.BICUBIC, box)
                 tiled.paste(tile, (x0, y0))
         return tiled
 
     def test_tiles(self):
-        im = Image.open("Tests/images/flower.jpg")
-        self.assertEqual(im.size, (480, 360))
-        dst_size = (251, 188)
-        reference = im.resize(dst_size, Image.BICUBIC)
+        with Image.open("Tests/images/flower.jpg") as im:
+            self.assertEqual(im.size, (480, 360))
+            dst_size = (251, 188)
+            reference = im.resize(dst_size, Image.BICUBIC)
 
-        for tiles in [(1, 1), (3, 3), (9, 7), (100, 100)]:
-            tiled = self.resize_tiled(im, dst_size, *tiles)
-            self.assert_image_similar(reference, tiled, 0.01)
+            for tiles in [(1, 1), (3, 3), (9, 7), (100, 100)]:
+                tiled = self.resize_tiled(im, dst_size, *tiles)
+                self.assert_image_similar(reference, tiled, 0.01)
 
     def test_subsample(self):
         # This test shows advantages of the subpixel resizing
         # after supersampling (e.g. during JPEG decoding).
-        im = Image.open("Tests/images/flower.jpg")
-        self.assertEqual(im.size, (480, 360))
-        dst_size = (48, 36)
-        # Reference is cropped image resized to destination
-        reference = im.crop((0, 0, 473, 353)).resize(dst_size, Image.BICUBIC)
-        # Image.BOX emulates supersampling (480 / 8 = 60, 360 / 8 = 45)
-        supersampled = im.resize((60, 45), Image.BOX)
+        with Image.open("Tests/images/flower.jpg") as im:
+            self.assertEqual(im.size, (480, 360))
+            dst_size = (48, 36)
+            # Reference is cropped image resized to destination
+            reference = im.crop((0, 0, 473, 353)).resize(dst_size, Image.BICUBIC)
+            # Image.BOX emulates supersampling (480 / 8 = 60, 360 / 8 = 45)
+            supersampled = im.resize((60, 45), Image.BOX)
 
-        with_box = supersampled.resize(dst_size, Image.BICUBIC,
-                                       (0, 0, 59.125, 44.125))
+        with_box = supersampled.resize(dst_size, Image.BICUBIC, (0, 0, 59.125, 44.125))
         without_box = supersampled.resize(dst_size, Image.BICUBIC)
 
         # error with box should be much smaller than without
@@ -458,7 +485,7 @@ class CoreResampleBoxTest(PillowTestCase):
 
     def test_formats(self):
         for resample in [Image.NEAREST, Image.BILINEAR]:
-            for mode in ['RGB', 'L', 'RGBA', 'LA', 'I', '']:
+            for mode in ["RGB", "L", "RGBA", "LA", "I", ""]:
                 im = hopper(mode)
                 box = (20, 20, im.size[0] - 20, im.size[1] - 20)
                 with_box = im.resize((32, 32), resample, box)
@@ -480,7 +507,7 @@ class CoreResampleBoxTest(PillowTestCase):
                 self.assertEqual(res.size, size)
                 self.assert_image_equal(res, im.crop(box))
             except AssertionError:
-                print('>>>', size, box)
+                print(">>>", size, box)
                 raise
 
     def test_no_passthrough(self):
@@ -500,7 +527,7 @@ class CoreResampleBoxTest(PillowTestCase):
                     # check that the difference at least that much
                     self.assert_image_similar(res, im.crop(box), 20)
             except AssertionError:
-                print('>>>', size, box)
+                print(">>>", size, box)
                 raise
 
     def test_skip_horizontal(self):
@@ -518,10 +545,9 @@ class CoreResampleBoxTest(PillowTestCase):
                     res = im.resize(size, flt, box)
                     self.assertEqual(res.size, size)
                     # Borders should be slightly different
-                    self.assert_image_similar(
-                        res, im.crop(box).resize(size, flt), 0.4)
+                    self.assert_image_similar(res, im.crop(box).resize(size, flt), 0.4)
                 except AssertionError:
-                    print('>>>', size, box, flt)
+                    print(">>>", size, box, flt)
                     raise
 
     def test_skip_vertical(self):
@@ -539,12 +565,7 @@ class CoreResampleBoxTest(PillowTestCase):
                     res = im.resize(size, flt, box)
                     self.assertEqual(res.size, size)
                     # Borders should be slightly different
-                    self.assert_image_similar(
-                        res, im.crop(box).resize(size, flt), 0.4)
+                    self.assert_image_similar(res, im.crop(box).resize(size, flt), 0.4)
                 except AssertionError:
-                    print('>>>', size, box, flt)
+                    print(">>>", size, box, flt)
                     raise
-
-
-if __name__ == '__main__':
-    unittest.main()
