@@ -1,8 +1,7 @@
-from helper import unittest, PillowTestCase, hopper, fromstring
+from .helper import PillowTestCase, fromstring, hopper
 
 
 class TestImageToBitmap(PillowTestCase):
-
     def test_sanity(self):
 
         self.assertRaises(ValueError, lambda: hopper().tobitmap())
@@ -13,7 +12,3 @@ class TestImageToBitmap(PillowTestCase):
 
         self.assertIsInstance(bitmap, bytes)
         self.assert_image_equal(im1, fromstring(bitmap))
-
-
-if __name__ == '__main__':
-    unittest.main()
