@@ -1,14 +1,12 @@
 import math
 
-from .helper import PillowTestCase, hopper
+from PIL import Image, ImageTransform
 
-from PIL import Image
+from .helper import PillowTestCase, hopper
 
 
 class TestImageTransform(PillowTestCase):
     def test_sanity(self):
-        from PIL import ImageTransform
-
         im = Image.new("L", (100, 100))
 
         seq = tuple(range(10))
@@ -22,6 +20,16 @@ class TestImageTransform(PillowTestCase):
         transform = ImageTransform.MeshTransform([(seq[:4], seq[:8])])
         im.transform((100, 100), transform)
 
+    def test_info(self):
+        comment = b"File written by Adobe Photoshop\xa8 4.0"
+
+        with Image.open("Tests/images/hopper.gif") as im:
+            self.assertEqual(im.info["comment"], comment)
+
+            transform = ImageTransform.ExtentTransform((0, 0, 0, 0))
+            new_im = im.transform((100, 100), transform)
+        self.assertEqual(new_im.info["comment"], comment)
+
     def test_extent(self):
         im = hopper("RGB")
         (w, h) = im.size
@@ -156,21 +164,21 @@ class TestImageTransform(PillowTestCase):
         self.test_mesh()
 
     def test_missing_method_data(self):
-        im = hopper()
-        self.assertRaises(ValueError, im.transform, (100, 100), None)
+        with hopper() as im:
+            self.assertRaises(ValueError, im.transform, (100, 100), None)
 
     def test_unknown_resampling_filter(self):
-        im = hopper()
-        (w, h) = im.size
-        for resample in (Image.BOX, "unknown"):
-            self.assertRaises(
-                ValueError,
-                im.transform,
-                (100, 100),
-                Image.EXTENT,
-                (0, 0, w, h),
-                resample,
-            )
+        with hopper() as im:
+            (w, h) = im.size
+            for resample in (Image.BOX, "unknown"):
+                self.assertRaises(
+                    ValueError,
+                    im.transform,
+                    (100, 100),
+                    Image.EXTENT,
+                    (0, 0, w, h),
+                    resample,
+                )
 
 
 class TestImageTransformAffine(PillowTestCase):
