@@ -1,7 +1,9 @@
 import os.path
+import unittest
+
+from PIL import Image, ImageColor, ImageDraw, ImageFont, features
 
 from .helper import PillowTestCase, hopper
-from PIL import Image, ImageColor, ImageDraw
 
 BLACK = (0, 0, 0)
 WHITE = (255, 255, 255)
@@ -28,6 +30,8 @@ POINTS2 = [10, 10, 20, 40, 30, 30]
 
 KITE_POINTS = [(10, 50), (70, 10), (90, 50), (70, 90), (10, 50)]
 
+HAS_FREETYPE = features.check("freetype2")
+
 
 class TestImageDraw(PillowTestCase):
     def test_sanity(self):
@@ -42,10 +46,10 @@ class TestImageDraw(PillowTestCase):
         draw.rectangle(list(range(4)))
 
     def test_valueerror(self):
-        im = Image.open("Tests/images/chi.gif")
+        with Image.open("Tests/images/chi.gif") as im:
 
-        draw = ImageDraw.Draw(im)
-        draw.line((0, 0), fill=(0, 0, 0))
+            draw = ImageDraw.Draw(im)
+            draw.line((0, 0), fill=(0, 0, 0))
 
     def test_mode_mismatch(self):
         im = hopper("RGB").copy()
@@ -139,14 +143,27 @@ class TestImageDraw(PillowTestCase):
         # Assert
         self.assert_image_similar(im, Image.open(expected), 1)
 
-    def test_bitmap(self):
+    def test_arc_width_non_whole_angle(self):
         # Arrange
-        small = Image.open("Tests/images/pil123rgba.png").resize((50, 50))
         im = Image.new("RGB", (W, H))
         draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_arc_width_non_whole_angle.png"
 
         # Act
-        draw.bitmap((10, 10), small)
+        draw.arc(BBOX1, 10, 259.5, width=5)
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 1)
+
+    def test_bitmap(self):
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        with Image.open("Tests/images/pil123rgba.png") as small:
+            small = small.resize((50, 50), Image.NEAREST)
+
+            # Act
+            draw.bitmap((10, 10), small)
 
         # Assert
         self.assert_image_equal(im, Image.open("Tests/images/imagedraw_bitmap.png"))
@@ -507,8 +524,8 @@ class TestImageDraw(PillowTestCase):
 
             # Assert
             expected = "Tests/images/imagedraw_floodfill_" + mode + ".png"
-            im_floodfill = Image.open(expected)
-            self.assert_image_equal(im, im_floodfill)
+            with Image.open(expected) as im_floodfill:
+                self.assert_image_equal(im, im_floodfill)
 
         # Test that using the same colour does not change the image
         ImageDraw.floodfill(im, centre_point, red)
@@ -558,6 +575,24 @@ class TestImageDraw(PillowTestCase):
         # Assert
         self.assert_image_equal(im, Image.open("Tests/images/imagedraw_floodfill2.png"))
 
+    def test_floodfill_not_negative(self):
+        # floodfill() is experimental
+        # Test that floodfill does not extend into negative coordinates
+
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        draw.line((W / 2, 0, W / 2, H / 2), fill="green")
+        draw.line((0, H / 2, W / 2, H / 2), fill="green")
+
+        # Act
+        ImageDraw.floodfill(im, (int(W / 4), int(H / 4)), ImageColor.getrgb("red"))
+
+        # Assert
+        self.assert_image_equal(
+            im, Image.open("Tests/images/imagedraw_floodfill_not_negative.png")
+        )
+
     def create_base_image_draw(
         self, size, mode=DEFAULT_MODE, background1=WHITE, background2=GRAY
     ):
@@ -569,152 +604,166 @@ class TestImageDraw(PillowTestCase):
         return img, ImageDraw.Draw(img)
 
     def test_square(self):
-        expected = Image.open(os.path.join(IMAGES_PATH, "square.png"))
-        expected.load()
-        img, draw = self.create_base_image_draw((10, 10))
-        draw.polygon([(2, 2), (2, 7), (7, 7), (7, 2)], BLACK)
-        self.assert_image_equal(img, expected, "square as normal polygon failed")
-        img, draw = self.create_base_image_draw((10, 10))
-        draw.polygon([(7, 7), (7, 2), (2, 2), (2, 7)], BLACK)
-        self.assert_image_equal(img, expected, "square as inverted polygon failed")
-        img, draw = self.create_base_image_draw((10, 10))
-        draw.rectangle((2, 2, 7, 7), BLACK)
-        self.assert_image_equal(img, expected, "square as normal rectangle failed")
-        img, draw = self.create_base_image_draw((10, 10))
-        draw.rectangle((7, 7, 2, 2), BLACK)
-        self.assert_image_equal(img, expected, "square as inverted rectangle failed")
+        with Image.open(os.path.join(IMAGES_PATH, "square.png")) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((10, 10))
+            draw.polygon([(2, 2), (2, 7), (7, 7), (7, 2)], BLACK)
+            self.assert_image_equal(img, expected, "square as normal polygon failed")
+            img, draw = self.create_base_image_draw((10, 10))
+            draw.polygon([(7, 7), (7, 2), (2, 2), (2, 7)], BLACK)
+            self.assert_image_equal(img, expected, "square as inverted polygon failed")
+            img, draw = self.create_base_image_draw((10, 10))
+            draw.rectangle((2, 2, 7, 7), BLACK)
+            self.assert_image_equal(img, expected, "square as normal rectangle failed")
+            img, draw = self.create_base_image_draw((10, 10))
+            draw.rectangle((7, 7, 2, 2), BLACK)
+            self.assert_image_equal(
+                img, expected, "square as inverted rectangle failed"
+            )
 
     def test_triangle_right(self):
-        expected = Image.open(os.path.join(IMAGES_PATH, "triangle_right.png"))
-        expected.load()
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.polygon([(3, 5), (17, 5), (10, 12)], BLACK)
-        self.assert_image_equal(img, expected, "triangle right failed")
+        with Image.open(os.path.join(IMAGES_PATH, "triangle_right.png")) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.polygon([(3, 5), (17, 5), (10, 12)], BLACK)
+            self.assert_image_equal(img, expected, "triangle right failed")
 
     def test_line_horizontal(self):
-        expected = Image.open(
+        with Image.open(
             os.path.join(IMAGES_PATH, "line_horizontal_w2px_normal.png")
-        )
-        expected.load()
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((5, 5, 14, 5), BLACK, 2)
-        self.assert_image_equal(
-            img, expected, "line straight horizontal normal 2px wide failed"
-        )
-        expected = Image.open(
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((5, 5, 14, 5), BLACK, 2)
+            self.assert_image_equal(
+                img, expected, "line straight horizontal normal 2px wide failed"
+            )
+        with Image.open(
             os.path.join(IMAGES_PATH, "line_horizontal_w2px_inverted.png")
-        )
-        expected.load()
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((14, 5, 5, 5), BLACK, 2)
-        self.assert_image_equal(
-            img, expected, "line straight horizontal inverted 2px wide failed"
-        )
-        expected = Image.open(os.path.join(IMAGES_PATH, "line_horizontal_w3px.png"))
-        expected.load()
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((5, 5, 14, 5), BLACK, 3)
-        self.assert_image_equal(
-            img, expected, "line straight horizontal normal 3px wide failed"
-        )
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((14, 5, 5, 5), BLACK, 3)
-        self.assert_image_equal(
-            img, expected, "line straight horizontal inverted 3px wide failed"
-        )
-        expected = Image.open(os.path.join(IMAGES_PATH, "line_horizontal_w101px.png"))
-        expected.load()
-        img, draw = self.create_base_image_draw((200, 110))
-        draw.line((5, 55, 195, 55), BLACK, 101)
-        self.assert_image_equal(
-            img, expected, "line straight horizontal 101px wide failed"
-        )
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((14, 5, 5, 5), BLACK, 2)
+            self.assert_image_equal(
+                img, expected, "line straight horizontal inverted 2px wide failed"
+            )
+        with Image.open(
+            os.path.join(IMAGES_PATH, "line_horizontal_w3px.png")
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((5, 5, 14, 5), BLACK, 3)
+            self.assert_image_equal(
+                img, expected, "line straight horizontal normal 3px wide failed"
+            )
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((14, 5, 5, 5), BLACK, 3)
+            self.assert_image_equal(
+                img, expected, "line straight horizontal inverted 3px wide failed"
+            )
+        with Image.open(
+            os.path.join(IMAGES_PATH, "line_horizontal_w101px.png")
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((200, 110))
+            draw.line((5, 55, 195, 55), BLACK, 101)
+            self.assert_image_equal(
+                img, expected, "line straight horizontal 101px wide failed"
+            )
 
     def test_line_h_s1_w2(self):
         self.skipTest("failing")
-        expected = Image.open(
+        with Image.open(
             os.path.join(IMAGES_PATH, "line_horizontal_slope1px_w2px.png")
-        )
-        expected.load()
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((5, 5, 14, 6), BLACK, 2)
-        self.assert_image_equal(
-            img, expected, "line horizontal 1px slope 2px wide failed"
-        )
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((5, 5, 14, 6), BLACK, 2)
+            self.assert_image_equal(
+                img, expected, "line horizontal 1px slope 2px wide failed"
+            )
 
     def test_line_vertical(self):
-        expected = Image.open(
+        with Image.open(
             os.path.join(IMAGES_PATH, "line_vertical_w2px_normal.png")
-        )
-        expected.load()
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((5, 5, 5, 14), BLACK, 2)
-        self.assert_image_equal(
-            img, expected, "line straight vertical normal 2px wide failed"
-        )
-        expected = Image.open(
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((5, 5, 5, 14), BLACK, 2)
+            self.assert_image_equal(
+                img, expected, "line straight vertical normal 2px wide failed"
+            )
+        with Image.open(
             os.path.join(IMAGES_PATH, "line_vertical_w2px_inverted.png")
-        )
-        expected.load()
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((5, 14, 5, 5), BLACK, 2)
-        self.assert_image_equal(
-            img, expected, "line straight vertical inverted 2px wide failed"
-        )
-        expected = Image.open(os.path.join(IMAGES_PATH, "line_vertical_w3px.png"))
-        expected.load()
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((5, 5, 5, 14), BLACK, 3)
-        self.assert_image_equal(
-            img, expected, "line straight vertical normal 3px wide failed"
-        )
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((5, 14, 5, 5), BLACK, 3)
-        self.assert_image_equal(
-            img, expected, "line straight vertical inverted 3px wide failed"
-        )
-        expected = Image.open(os.path.join(IMAGES_PATH, "line_vertical_w101px.png"))
-        expected.load()
-        img, draw = self.create_base_image_draw((110, 200))
-        draw.line((55, 5, 55, 195), BLACK, 101)
-        self.assert_image_equal(
-            img, expected, "line straight vertical 101px wide failed"
-        )
-        expected = Image.open(
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((5, 14, 5, 5), BLACK, 2)
+            self.assert_image_equal(
+                img, expected, "line straight vertical inverted 2px wide failed"
+            )
+        with Image.open(
+            os.path.join(IMAGES_PATH, "line_vertical_w3px.png")
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((5, 5, 5, 14), BLACK, 3)
+            self.assert_image_equal(
+                img, expected, "line straight vertical normal 3px wide failed"
+            )
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((5, 14, 5, 5), BLACK, 3)
+            self.assert_image_equal(
+                img, expected, "line straight vertical inverted 3px wide failed"
+            )
+        with Image.open(
+            os.path.join(IMAGES_PATH, "line_vertical_w101px.png")
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((110, 200))
+            draw.line((55, 5, 55, 195), BLACK, 101)
+            self.assert_image_equal(
+                img, expected, "line straight vertical 101px wide failed"
+            )
+        with Image.open(
             os.path.join(IMAGES_PATH, "line_vertical_slope1px_w2px.png")
-        )
-        expected.load()
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((5, 5, 6, 14), BLACK, 2)
-        self.assert_image_equal(
-            img, expected, "line vertical 1px slope 2px wide failed"
-        )
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((5, 5, 6, 14), BLACK, 2)
+            self.assert_image_equal(
+                img, expected, "line vertical 1px slope 2px wide failed"
+            )
 
     def test_line_oblique_45(self):
-        expected = Image.open(os.path.join(IMAGES_PATH, "line_oblique_45_w3px_a.png"))
-        expected.load()
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((5, 5, 14, 14), BLACK, 3)
-        self.assert_image_equal(
-            img, expected, "line oblique 45 normal 3px wide A failed"
-        )
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((14, 14, 5, 5), BLACK, 3)
-        self.assert_image_equal(
-            img, expected, "line oblique 45 inverted 3px wide A failed"
-        )
-        expected = Image.open(os.path.join(IMAGES_PATH, "line_oblique_45_w3px_b.png"))
-        expected.load()
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((14, 5, 5, 14), BLACK, 3)
-        self.assert_image_equal(
-            img, expected, "line oblique 45 normal 3px wide B failed"
-        )
-        img, draw = self.create_base_image_draw((20, 20))
-        draw.line((5, 14, 14, 5), BLACK, 3)
-        self.assert_image_equal(
-            img, expected, "line oblique 45 inverted 3px wide B failed"
-        )
+        with Image.open(
+            os.path.join(IMAGES_PATH, "line_oblique_45_w3px_a.png")
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((5, 5, 14, 14), BLACK, 3)
+            self.assert_image_equal(
+                img, expected, "line oblique 45 normal 3px wide A failed"
+            )
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((14, 14, 5, 5), BLACK, 3)
+            self.assert_image_equal(
+                img, expected, "line oblique 45 inverted 3px wide A failed"
+            )
+        with Image.open(
+            os.path.join(IMAGES_PATH, "line_oblique_45_w3px_b.png")
+        ) as expected:
+            expected.load()
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((14, 5, 5, 14), BLACK, 3)
+            self.assert_image_equal(
+                img, expected, "line oblique 45 normal 3px wide B failed"
+            )
+            img, draw = self.create_base_image_draw((20, 20))
+            draw.line((5, 14, 14, 5), BLACK, 3)
+            self.assert_image_equal(
+                img, expected, "line oblique 45 inverted 3px wide B failed"
+            )
 
     def test_wide_line_dot(self):
         # Test drawing a wide "line" from one point to another just draws
@@ -770,6 +819,54 @@ class TestImageDraw(PillowTestCase):
         draw.textsize("\n")
         draw.textsize("test\n")
 
+    @unittest.skipUnless(HAS_FREETYPE, "ImageFont not available")
+    def test_textsize_stroke(self):
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        font = ImageFont.truetype("Tests/fonts/FreeMono.ttf", 20)
+
+        # Act / Assert
+        self.assertEqual(draw.textsize("A", font, stroke_width=2), (16, 20))
+        self.assertEqual(
+            draw.multiline_textsize("ABC\nAaaa", font, stroke_width=2), (52, 44)
+        )
+
+    @unittest.skipUnless(HAS_FREETYPE, "ImageFont not available")
+    def test_stroke(self):
+        for suffix, stroke_fill in {"same": None, "different": "#0f0"}.items():
+            # Arrange
+            im = Image.new("RGB", (120, 130))
+            draw = ImageDraw.Draw(im)
+            font = ImageFont.truetype("Tests/fonts/FreeMono.ttf", 120)
+
+            # Act
+            draw.text(
+                (10, 10), "A", "#f00", font, stroke_width=2, stroke_fill=stroke_fill
+            )
+
+            # Assert
+            self.assert_image_similar(
+                im, Image.open("Tests/images/imagedraw_stroke_" + suffix + ".png"), 3.1
+            )
+
+    @unittest.skipUnless(HAS_FREETYPE, "ImageFont not available")
+    def test_stroke_multiline(self):
+        # Arrange
+        im = Image.new("RGB", (100, 250))
+        draw = ImageDraw.Draw(im)
+        font = ImageFont.truetype("Tests/fonts/FreeMono.ttf", 120)
+
+        # Act
+        draw.multiline_text(
+            (10, 10), "A\nB", "#f00", font, stroke_width=2, stroke_fill="#0f0"
+        )
+
+        # Assert
+        self.assert_image_similar(
+            im, Image.open("Tests/images/imagedraw_stroke_multiline.png"), 3.3
+        )
+
     def test_same_color_outline(self):
         # Prepare shape
         x0, y0 = 5, 5
