@@ -1,7 +1,6 @@
-from .helper import PillowTestCase, hopper
+from PIL import Image, ImageEnhance
 
-from PIL import Image
-from PIL import ImageEnhance
+from .helper import PillowTestCase, hopper
 
 
 class TestImageEnhance(PillowTestCase):
@@ -36,7 +35,7 @@ class TestImageEnhance(PillowTestCase):
         self.assert_image_equal(
             im.getchannel("A"),
             original.getchannel("A"),
-            "Diff on %s: %s" % (op, amount),
+            "Diff on {}: {}".format(op, amount),
         )
 
     def test_alpha(self):
