@@ -1,7 +1,8 @@
-from .helper import unittest, PillowTestCase, hopper
+import unittest
 
 from PIL import ImageWin
-import sys
+
+from .helper import PillowTestCase, hopper, is_win32
 
 
 class TestImageWin(PillowTestCase):
@@ -31,7 +32,7 @@ class TestImageWin(PillowTestCase):
         self.assertEqual(wnd2, 50)
 
 
-@unittest.skipUnless(sys.platform.startswith("win32"), "Windows only")
+@unittest.skipUnless(is_win32(), "Windows only")
 class TestImageWinDib(PillowTestCase):
     def test_dib_image(self):
         # Arrange
