@@ -1,10 +1,11 @@
-from helper import unittest, PillowTestCase
+import unittest
 
 from PIL import Image
 
+from .helper import PillowTestCase
+
 
 class TestLibImage(PillowTestCase):
-
     def test_setmode(self):
 
         im = Image.new("L", (1, 1), 255)
@@ -33,5 +34,5 @@ class TestLibImage(PillowTestCase):
         self.assertRaises(ValueError, im.im.setmode, "RGBABCDE")
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
