@@ -1,4 +1,4 @@
-from .helper import PillowTestCase
+import time
 
 from PIL.PdfParser import (
     IndirectObjectDef,
@@ -13,7 +13,8 @@ from PIL.PdfParser import (
     encode_text,
     pdf_repr,
 )
-import time
+
+from .helper import PillowTestCase
 
 
 class TestPdfParser(PillowTestCase):
@@ -21,7 +22,7 @@ class TestPdfParser(PillowTestCase):
         self.assertEqual(encode_text("abc"), b"\xFE\xFF\x00a\x00b\x00c")
         self.assertEqual(decode_text(b"\xFE\xFF\x00a\x00b\x00c"), "abc")
         self.assertEqual(decode_text(b"abc"), "abc")
-        self.assertEqual(decode_text(b"\x1B a \x1C"), u"\u02D9 a \u02DD")
+        self.assertEqual(decode_text(b"\x1B a \x1C"), "\u02D9 a \u02DD")
 
     def test_indirect_refs(self):
         self.assertEqual(IndirectReference(1, 2), IndirectReference(1, 2))
