@@ -1,7 +1,9 @@
-from .helper import unittest, PillowTestCase
+import unittest
 
 from PIL import __version__
 
+from .helper import PillowTestCase
+
 try:
     import pyroma
 except ImportError:
