@@ -1,34 +1,17 @@
+from PIL import Image, ImageQt
+
 from .helper import PillowTestCase, hopper
 from .test_imageqt import PillowQtTestCase
 
-from PIL import ImageQt, Image
-
-
 if ImageQt.qt_is_installed:
     from PIL.ImageQt import QImage
 
     try:
         from PyQt5 import QtGui
         from PyQt5.QtWidgets import QWidget, QHBoxLayout, QLabel, QApplication
-
-        QT_VERSION = 5
     except (ImportError, RuntimeError):
-        try:
-            from PySide2 import QtGui
-            from PySide2.QtWidgets import QWidget, QHBoxLayout, QLabel, QApplication
-
-            QT_VERSION = 5
-        except (ImportError, RuntimeError):
-            try:
-                from PyQt4 import QtGui
-                from PyQt4.QtGui import QWidget, QHBoxLayout, QLabel, QApplication
-
-                QT_VERSION = 4
-            except (ImportError, RuntimeError):
-                from PySide import QtGui
-                from PySide.QtGui import QWidget, QHBoxLayout, QLabel, QApplication
-
-                QT_VERSION = 4
+        from PySide2 import QtGui
+        from PySide2.QtWidgets import QWidget, QHBoxLayout, QLabel, QApplication
 
 
 class TestToQImage(PillowQtTestCase, PillowTestCase):
@@ -60,12 +43,8 @@ class TestToQImage(PillowQtTestCase, PillowTestCase):
             data.save(tempfile)
 
             # Check that it actually worked.
-            reloaded = Image.open(tempfile)
-            # Gray images appear to come back in palette mode.
-            # They're roughly equivalent
-            if QT_VERSION == 4 and mode == "L":
-                src = src.convert("P")
-            self.assert_image_equal(reloaded, src)
+            with Image.open(tempfile) as reloaded:
+                self.assert_image_equal(reloaded, src)
 
     def test_segfault(self):
         app = QApplication([])
@@ -78,7 +57,7 @@ if ImageQt.qt_is_installed:
 
     class Example(QWidget):
         def __init__(self):
-            super(Example, self).__init__()
+            super().__init__()
 
             img = hopper().resize((1000, 1000))
 
