@@ -1,7 +1,10 @@
-from .helper import unittest, PillowLeakTestCase
-from PIL import Image, features
+import unittest
 from io import BytesIO
 
+from PIL import Image, features
+
+from .helper import PillowLeakTestCase
+
 test_file = "Tests/images/hopper.webp"
 
 
