@@ -24,17 +24,17 @@
 # See the README file for information on usage and redistribution.
 #
 
-from . import Image, ImageFile, ImagePalette, ImageChops, ImageSequence
-from ._binary import i8, i16le as i16, o8, o16le as o16
-
 import itertools
+import os
+import subprocess
 
-__version__ = "0.9"
-
+from . import Image, ImageChops, ImageFile, ImagePalette, ImageSequence
+from ._binary import i8, i16le as i16, o8, o16le as o16
 
 # --------------------------------------------------------------------
 # Identify/read GIF files
 
+
 def _accept(prefix):
     return prefix[:6] in [b"GIF87a", b"GIF89a"]
 
@@ -43,6 +43,7 @@ def _accept(prefix):
 # Image plugin for GIF images.  This plugin supports both GIF87 and
 # GIF89 images.
 
+
 class GifImageFile(ImageFile.ImageFile):
 
     format = "GIF"
@@ -76,7 +77,7 @@ class GifImageFile(ImageFile.ImageFile):
             # check if palette contains colour indices
             p = self.fp.read(3 << bits)
             for i in range(0, len(p), 3):
-                if not (i//3 == i8(p[i]) == i8(p[i+1]) == i8(p[i+2])):
+                if not (i // 3 == i8(p[i]) == i8(p[i + 1]) == i8(p[i + 2])):
                     p = ImagePalette.raw("RGB", p)
                     self.global_palette = self.palette = p
                     break
@@ -120,6 +121,8 @@ class GifImageFile(ImageFile.ImageFile):
         if not self._seek_check(frame):
             return
         if frame < self.__frame:
+            if frame != 0:
+                self.im = None
             self._seek(0)
 
         last_frame = self.__frame
@@ -164,6 +167,7 @@ class GifImageFile(ImageFile.ImageFile):
             self.im.paste(self.dispose, self.dispose_extent)
 
         from copy import copy
+
         self.palette = copy(self.global_palette)
 
         info = {}
@@ -229,6 +233,8 @@ class GifImageFile(ImageFile.ImageFile):
                 # extent
                 x0, y0 = i16(s[0:]), i16(s[2:])
                 x1, y1 = x0 + i16(s[4:]), y0 + i16(s[6:])
+                if x1 > self.size[0] or y1 > self.size[1]:
+                    self._size = max(x1, self.size[0]), max(y1, self.size[1])
                 self.dispose_extent = x0, y0, x1, y1
                 flags = i8(s[8])
 
@@ -236,16 +242,14 @@ class GifImageFile(ImageFile.ImageFile):
 
                 if flags & 128:
                     bits = (flags & 7) + 1
-                    self.palette =\
-                        ImagePalette.raw("RGB", self.fp.read(3 << bits))
+                    self.palette = ImagePalette.raw("RGB", self.fp.read(3 << bits))
 
                 # image data
                 bits = i8(self.fp.read(1))
                 self.__offset = self.fp.tell()
-                self.tile = [("gif",
-                             (x0, y0, x1, y1),
-                             self.__offset,
-                             (bits, interlace))]
+                self.tile = [
+                    ("gif", (x0, y0, x1, y1), self.__offset, (bits, interlace))
+                ]
                 break
 
             else:
@@ -258,8 +262,8 @@ class GifImageFile(ImageFile.ImageFile):
                 self.dispose = None
             elif self.disposal_method == 2:
                 # replace with background colour
-                self.dispose = Image.core.fill("P", self.size,
-                                               self.info["background"])
+                Image._decompression_bomb_check(self.size)
+                self.dispose = Image.core.fill("P", self.size, self.info["background"])
             else:
                 # replace with previous contents
                 if self.im:
@@ -297,8 +301,7 @@ class GifImageFile(ImageFile.ImageFile):
             # we do this by pasting the updated area onto the previous
             # frame which we then use as the current image content
             updated = self._crop(self.im, self.dispose_extent)
-            self._prev_im.paste(updated, self.dispose_extent,
-                                updated.convert('RGBA'))
+            self._prev_im.paste(updated, self.dispose_extent, updated.convert("RGBA"))
             self.im = self._prev_im
         self._prev_im = self.im.copy()
 
@@ -311,15 +314,12 @@ class GifImageFile(ImageFile.ImageFile):
         finally:
             self.__fp = None
 
+
 # --------------------------------------------------------------------
 # Write GIF files
 
 
-RAWMODE = {
-    "1": "L",
-    "L": "L",
-    "P": "P"
-}
+RAWMODE = {"1": "L", "L": "L", "P": "P"}
 
 
 def _normalize_mode(im, initial_call=False):
@@ -370,19 +370,23 @@ def _normalize_palette(im, palette, info):
         if isinstance(palette, (bytes, bytearray, list)):
             source_palette = bytearray(palette[:768])
         if isinstance(palette, ImagePalette.ImagePalette):
-            source_palette = bytearray(itertools.chain.from_iterable(
-                                zip(palette.palette[:256],
-                                    palette.palette[256:512],
-                                    palette.palette[512:768])))
+            source_palette = bytearray(
+                itertools.chain.from_iterable(
+                    zip(
+                        palette.palette[:256],
+                        palette.palette[256:512],
+                        palette.palette[512:768],
+                    )
+                )
+            )
 
     if im.mode == "P":
         if not source_palette:
             source_palette = im.im.getpalette("RGB")[:768]
     else:  # L-mode
         if not source_palette:
-            source_palette = bytearray(i//3 for i in range(768))
-        im.palette = ImagePalette.ImagePalette("RGB",
-                                               palette=source_palette)
+            source_palette = bytearray(i // 3 for i in range(768))
+        im.palette = ImagePalette.ImagePalette("RGB", palette=source_palette)
 
     used_palette_colors = _get_optimize(im, info)
     if used_palette_colors is not None:
@@ -408,8 +412,7 @@ def _write_single_frame(im, fp, palette):
     _write_local_header(fp, im, (0, 0), flags)
 
     im_out.encoderconfig = (8, get_interlace(im))
-    ImageFile._save(im_out, fp, [("gif", (0, 0)+im.size, 0,
-                                  RAWMODE[im_out.mode])])
+    ImageFile._save(im_out, fp, [("gif", (0, 0) + im.size, 0, RAWMODE[im_out.mode])])
 
     fp.write(b"\0")  # end of image data
 
@@ -421,8 +424,8 @@ def _write_multiple_frames(im, fp, palette):
 
     im_frames = []
     frame_count = 0
-    for imSequence in itertools.chain([im],
-                                      im.encoderinfo.get("append_images", [])):
+    background_im = None
+    for imSequence in itertools.chain([im], im.encoderinfo.get("append_images", [])):
         for im_frame in ImageSequence.Iterator(imSequence):
             # a copy is required here since seek can still mutate the image
             im_frame = _normalize_mode(im_frame.copy())
@@ -433,7 +436,7 @@ def _write_multiple_frames(im, fp, palette):
 
             encoderinfo = im.encoderinfo.copy()
             if isinstance(duration, (list, tuple)):
-                encoderinfo['duration'] = duration[frame_count]
+                encoderinfo["duration"] = duration[frame_count]
             if isinstance(disposal, (list, tuple)):
                 encoderinfo["disposal"] = disposal[frame_count]
             frame_count += 1
@@ -441,45 +444,54 @@ def _write_multiple_frames(im, fp, palette):
             if im_frames:
                 # delta frame
                 previous = im_frames[-1]
-                if _get_palette_bytes(im_frame) == \
-                   _get_palette_bytes(previous['im']):
-                    delta = ImageChops.subtract_modulo(im_frame,
-                                                       previous['im'])
+                if encoderinfo.get("disposal") == 2:
+                    if background_im is None:
+                        background = _get_background(
+                            im,
+                            im.encoderinfo.get("background", im.info.get("background")),
+                        )
+                        background_im = Image.new("P", im_frame.size, background)
+                        background_im.putpalette(im_frames[0]["im"].palette)
+                    base_im = background_im
+                else:
+                    base_im = previous["im"]
+                if _get_palette_bytes(im_frame) == _get_palette_bytes(base_im):
+                    delta = ImageChops.subtract_modulo(im_frame, base_im)
                 else:
                     delta = ImageChops.subtract_modulo(
-                        im_frame.convert('RGB'), previous['im'].convert('RGB'))
+                        im_frame.convert("RGB"), base_im.convert("RGB")
+                    )
                 bbox = delta.getbbox()
                 if not bbox:
                     # This frame is identical to the previous frame
                     if duration:
-                        previous['encoderinfo']['duration'] += \
-                            encoderinfo['duration']
+                        previous["encoderinfo"]["duration"] += encoderinfo["duration"]
                     continue
             else:
                 bbox = None
-            im_frames.append({
-                'im': im_frame,
-                'bbox': bbox,
-                'encoderinfo': encoderinfo
-            })
+            im_frames.append({"im": im_frame, "bbox": bbox, "encoderinfo": encoderinfo})
 
     if len(im_frames) > 1:
         for frame_data in im_frames:
-            im_frame = frame_data['im']
-            if not frame_data['bbox']:
+            im_frame = frame_data["im"]
+            if not frame_data["bbox"]:
                 # global header
-                for s in _get_global_header(im_frame,
-                                            frame_data['encoderinfo']):
+                for s in _get_global_header(im_frame, frame_data["encoderinfo"]):
                     fp.write(s)
                 offset = (0, 0)
             else:
                 # compress difference
-                frame_data['encoderinfo']['include_color_table'] = True
+                frame_data["encoderinfo"]["include_color_table"] = True
 
-                im_frame = im_frame.crop(frame_data['bbox'])
-                offset = frame_data['bbox'][:2]
-            _write_frame_data(fp, im_frame, offset, frame_data['encoderinfo'])
+                im_frame = im_frame.crop(frame_data["bbox"])
+                offset = frame_data["bbox"][:2]
+            _write_frame_data(fp, im_frame, offset, frame_data["encoderinfo"])
         return True
+    elif "duration" in im.encoderinfo and isinstance(
+        im.encoderinfo["duration"], (list, tuple)
+    ):
+        # Since multiple frames will not be written, add together the frame durations
+        im.encoderinfo["duration"] = sum(im.encoderinfo["duration"])
 
 
 def _save_all(im, fp, filename):
@@ -537,7 +549,7 @@ def _write_local_header(fp, im, offset, flags):
     else:
         duration = 0
 
-    disposal = int(im.encoderinfo.get('disposal', 0))
+    disposal = int(im.encoderinfo.get("disposal", 0))
 
     if transparent_color_exists or duration != 0 or disposal:
         packed_flag = 1 if transparent_color_exists else 0
@@ -545,50 +557,56 @@ def _write_local_header(fp, im, offset, flags):
         if not transparent_color_exists:
             transparency = 0
 
-        fp.write(b"!" +
-                 o8(249) +                # extension intro
-                 o8(4) +                  # length
-                 o8(packed_flag) +        # packed fields
-                 o16(duration) +          # duration
-                 o8(transparency) +       # transparency index
-                 o8(0))
+        fp.write(
+            b"!"
+            + o8(249)  # extension intro
+            + o8(4)  # length
+            + o8(packed_flag)  # packed fields
+            + o16(duration)  # duration
+            + o8(transparency)  # transparency index
+            + o8(0)
+        )
 
-    if "comment" in im.encoderinfo and \
-       1 <= len(im.encoderinfo["comment"]):
-        fp.write(b"!" +
-                 o8(254))                 # extension intro
-        for i in range(0, len(im.encoderinfo["comment"]), 255):
-            subblock = im.encoderinfo["comment"][i:i+255]
-            fp.write(o8(len(subblock)) +
-                     subblock)
+    if "comment" in im.encoderinfo and 1 <= len(im.encoderinfo["comment"]):
+        fp.write(b"!" + o8(254))  # extension intro
+        comment = im.encoderinfo["comment"]
+        if isinstance(comment, str):
+            comment = comment.encode()
+        for i in range(0, len(comment), 255):
+            subblock = comment[i : i + 255]
+            fp.write(o8(len(subblock)) + subblock)
         fp.write(o8(0))
     if "loop" in im.encoderinfo:
         number_of_loops = im.encoderinfo["loop"]
-        fp.write(b"!" +
-                 o8(255) +                # extension intro
-                 o8(11) +
-                 b"NETSCAPE2.0" +
-                 o8(3) +
-                 o8(1) +
-                 o16(number_of_loops) +   # number of loops
-                 o8(0))
-    include_color_table = im.encoderinfo.get('include_color_table')
+        fp.write(
+            b"!"
+            + o8(255)  # extension intro
+            + o8(11)
+            + b"NETSCAPE2.0"
+            + o8(3)
+            + o8(1)
+            + o16(number_of_loops)  # number of loops
+            + o8(0)
+        )
+    include_color_table = im.encoderinfo.get("include_color_table")
     if include_color_table:
         palette_bytes = _get_palette_bytes(im)
         color_table_size = _get_color_table_size(palette_bytes)
         if color_table_size:
-            flags = flags | 128               # local color table flag
+            flags = flags | 128  # local color table flag
             flags = flags | color_table_size
 
-    fp.write(b"," +
-             o16(offset[0]) +             # offset
-             o16(offset[1]) +
-             o16(im.size[0]) +            # size
-             o16(im.size[1]) +
-             o8(flags))                   # flags
+    fp.write(
+        b","
+        + o16(offset[0])  # offset
+        + o16(offset[1])
+        + o16(im.size[0])  # size
+        + o16(im.size[1])
+        + o8(flags)  # flags
+    )
     if include_color_table and color_table_size:
         fp.write(_get_header_palette(palette_bytes))
-    fp.write(o8(8))                       # bits
+    fp.write(o8(8))  # bits
 
 
 def _save_netpbm(im, fp, filename):
@@ -599,40 +617,44 @@ def _save_netpbm(im, fp, filename):
     # If you need real GIF compression and/or RGB quantization, you
     # can use the external NETPBM/PBMPLUS utilities.  See comments
     # below for information on how to enable this.
-
-    import os
-    from subprocess import Popen, check_call, PIPE, CalledProcessError
-    file = im._dump()
-
-    with open(filename, 'wb') as f:
-        if im.mode != "RGB":
-            with open(os.devnull, 'wb') as devnull:
-                check_call(["ppmtogif", file], stdout=f, stderr=devnull)
-        else:
-            # Pipe ppmquant output into ppmtogif
-            # "ppmquant 256 %s | ppmtogif > %s" % (file, filename)
-            quant_cmd = ["ppmquant", "256", file]
-            togif_cmd = ["ppmtogif"]
-            with open(os.devnull, 'wb') as devnull:
-                quant_proc = Popen(quant_cmd, stdout=PIPE, stderr=devnull)
-                togif_proc = Popen(togif_cmd, stdin=quant_proc.stdout,
-                                   stdout=f, stderr=devnull)
-
-            # Allow ppmquant to receive SIGPIPE if ppmtogif exits
-            quant_proc.stdout.close()
-
-            retcode = quant_proc.wait()
-            if retcode:
-                raise CalledProcessError(retcode, quant_cmd)
-
-            retcode = togif_proc.wait()
-            if retcode:
-                raise CalledProcessError(retcode, togif_cmd)
+    tempfile = im._dump()
 
     try:
-        os.unlink(file)
-    except OSError:
-        pass
+        with open(filename, "wb") as f:
+            if im.mode != "RGB":
+                subprocess.check_call(
+                    ["ppmtogif", tempfile], stdout=f, stderr=subprocess.DEVNULL
+                )
+            else:
+                # Pipe ppmquant output into ppmtogif
+                # "ppmquant 256 %s | ppmtogif > %s" % (tempfile, filename)
+                quant_cmd = ["ppmquant", "256", tempfile]
+                togif_cmd = ["ppmtogif"]
+                quant_proc = subprocess.Popen(
+                    quant_cmd, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL
+                )
+                togif_proc = subprocess.Popen(
+                    togif_cmd,
+                    stdin=quant_proc.stdout,
+                    stdout=f,
+                    stderr=subprocess.DEVNULL,
+                )
+
+                # Allow ppmquant to receive SIGPIPE if ppmtogif exits
+                quant_proc.stdout.close()
+
+                retcode = quant_proc.wait()
+                if retcode:
+                    raise subprocess.CalledProcessError(retcode, quant_cmd)
+
+                retcode = togif_proc.wait()
+                if retcode:
+                    raise subprocess.CalledProcessError(retcode, togif_cmd)
+    finally:
+        try:
+            os.unlink(tempfile)
+        except OSError:
+            pass
 
 
 # Force optimization so that we can test performance against
@@ -662,7 +684,7 @@ def _get_optimize(im, info):
         # * If we have a 'large' image, the palette is in the noise.
 
         # create the new palette if not every color is used
-        optimise = _FORCE_OPTIMIZE or im.mode == 'L'
+        optimise = _FORCE_OPTIMIZE or im.mode == "L"
         if optimise or im.width * im.height < 512 * 512:
             # check which colors are used
             used_palette_colors = []
@@ -670,18 +692,23 @@ def _get_optimize(im, info):
                 if count:
                     used_palette_colors.append(i)
 
-            if optimise or (len(used_palette_colors) <= 128 and
-               max(used_palette_colors) > len(used_palette_colors)):
+            if optimise or (
+                len(used_palette_colors) <= 128
+                and max(used_palette_colors) > len(used_palette_colors)
+            ):
                 return used_palette_colors
 
 
 def _get_color_table_size(palette_bytes):
     # calculate the palette size for the header
     import math
-    color_table_size = int(math.ceil(math.log(len(palette_bytes)//3, 2)))-1
-    if color_table_size < 0:
-        color_table_size = 0
-    return color_table_size
+
+    if not palette_bytes:
+        return 0
+    elif len(palette_bytes) < 9:
+        return 1
+    else:
+        return int(math.ceil(math.log(len(palette_bytes) // 3, 2))) - 1
 
 
 def _get_header_palette(palette_bytes):
@@ -696,7 +723,7 @@ def _get_header_palette(palette_bytes):
 
     # add the missing amount of bytes
     # the palette has to be 2<<n in size
-    actual_target_size_diff = (2 << color_table_size) - len(palette_bytes)//3
+    actual_target_size_diff = (2 << color_table_size) - len(palette_bytes) // 3
     if actual_target_size_diff > 0:
         palette_bytes += o8(0) * 3 * actual_target_size_diff
     return palette_bytes
@@ -712,6 +739,18 @@ def _get_palette_bytes(im):
     return im.palette.palette
 
 
+def _get_background(im, infoBackground):
+    background = 0
+    if infoBackground:
+        background = infoBackground
+        if isinstance(background, tuple):
+            # WebPImagePlugin stores an RGBA value in info["background"]
+            # So it must be converted to the same format as GifImagePlugin's
+            # info["background"] - a global color table index
+            background = im.palette.getcolor(background)
+    return background
+
+
 def _get_global_header(im, info):
     """Return a list of strings representing a GIF header"""
 
@@ -721,9 +760,9 @@ def _get_global_header(im, info):
     version = b"87a"
     for extensionKey in ["transparency", "duration", "loop", "comment"]:
         if info and extensionKey in info:
-            if ((extensionKey == "duration" and info[extensionKey] == 0) or
-                (extensionKey == "comment" and
-                 not (1 <= len(info[extensionKey]) <= 255))):
+            if (extensionKey == "duration" and info[extensionKey] == 0) or (
+                extensionKey == "comment" and not (1 <= len(info[extensionKey]) <= 255)
+            ):
                 continue
             version = b"89a"
             break
@@ -731,31 +770,23 @@ def _get_global_header(im, info):
         if im.info.get("version") == b"89a":
             version = b"89a"
 
-    background = 0
-    if "background" in info:
-        background = info["background"]
-        if isinstance(background, tuple):
-            # WebPImagePlugin stores an RGBA value in info["background"]
-            # So it must be converted to the same format as GifImagePlugin's
-            # info["background"] - a global color table index
-            background = im.palette.getcolor(background)
+    background = _get_background(im, info.get("background"))
 
     palette_bytes = _get_palette_bytes(im)
     color_table_size = _get_color_table_size(palette_bytes)
 
     return [
-        b"GIF"+version +               # signature + version
-        o16(im.size[0]) +              # canvas width
-        o16(im.size[1]),               # canvas height
-
+        b"GIF"  # signature
+        + version  # version
+        + o16(im.size[0])  # canvas width
+        + o16(im.size[1]),  # canvas height
         # Logical Screen Descriptor
         # size of global color table + global color table flag
-        o8(color_table_size + 128),   # packed fields
+        o8(color_table_size + 128),  # packed fields
         # background + reserved/aspect
         o8(background) + o8(0),
-
         # Global Color Table
-        _get_header_palette(palette_bytes)
+        _get_header_palette(palette_bytes),
     ]
 
 
@@ -766,13 +797,15 @@ def _write_frame_data(fp, im_frame, offset, params):
         # local image header
         _write_local_header(fp, im_frame, offset, 0)
 
-        ImageFile._save(im_frame, fp, [("gif", (0, 0)+im_frame.size, 0,
-                                        RAWMODE[im_frame.mode])])
+        ImageFile._save(
+            im_frame, fp, [("gif", (0, 0) + im_frame.size, 0, RAWMODE[im_frame.mode])]
+        )
 
         fp.write(b"\0")  # end of image data
     finally:
         del im_frame.encoderinfo
 
+
 # --------------------------------------------------------------------
 # Legacy GIF utilities
 
@@ -821,7 +854,8 @@ def getdata(im, offset=(0, 0), **params):
     :returns: List of Bytes containing gif encoded frame data
 
     """
-    class Collector(object):
+
+    class Collector:
         data = []
 
         def write(self, data):
