@@ -16,7 +16,7 @@
  * following lines to your Tcl_AppInit function (in tkappinit.c)
  * instead.  Put them after the calls to Tcl_Init and Tk_Init:
  *
- *	{
+ *      {
  *          extern void TkImaging_Init(Tcl_Interp* interp);
  *          TkImaging_Init(interp);
  *      }
@@ -69,7 +69,7 @@ ImagingFind(const char* name)
     id = atol(name);
 #endif
     if (!id)
-	return NULL;
+        return NULL;
 
     return (Imaging) id;
 }
@@ -109,46 +109,29 @@ PyImagingPhotoPut(ClientData clientdata, Tcl_Interp* interp,
         return TCL_ERROR;
     }
 
-    /* Active region */
-#if 0
-    if (src_xoffset + xsize > im->xsize)
-	xsize = im->xsize - src_xoffset;
-    if (src_yoffset + ysize > im->ysize)
-	ysize = im->ysize - src_yoffset;
-    if (xsize < 0 || ysize < 0
-	|| src_xoffset >= im->xsize
-	|| src_yoffset >= im->ysize)
-	return TCL_OK;
-#endif
-
     /* Mode */
 
     if (strcmp(im->mode, "1") == 0 || strcmp(im->mode, "L") == 0) {
-	block.pixelSize = 1;
-	block.offset[0] = block.offset[1] = block.offset[2] = 0;
+        block.pixelSize = 1;
+        block.offset[0] = block.offset[1] = block.offset[2] = 0;
     } else if (strncmp(im->mode, "RGB", 3) == 0) {
-	block.pixelSize = 4;
-	block.offset[0] = 0;
-	block.offset[1] = 1;
-	block.offset[2] = 2;
+        block.pixelSize = 4;
+        block.offset[0] = 0;
+        block.offset[1] = 1;
+        block.offset[2] = 2;
         if (strcmp(im->mode, "RGBA") == 0)
             block.offset[3] = 3; /* alpha (or reserved, under 8.2) */
         else
             block.offset[3] = 0; /* no alpha */
     } else {
         TCL_APPEND_RESULT(interp, "Bad mode", (char*) NULL);
-	return TCL_ERROR;
+        return TCL_ERROR;
     }
 
     block.width = im->xsize;
     block.height = im->ysize;
     block.pitch = im->linesize;
     block.pixelPtr = (unsigned char*) im->block;
-#if 0
-    block.pixelPtr = (unsigned char*) im->block +
-	             src_yoffset * im->linesize +
-	             src_xoffset * im->pixelsize;
-#endif
 
     if (TK_LT_85) { /* Tk 8.4 */
         TK_PHOTO_PUT_BLOCK_84(photo, &block, 0, 0, block.width, block.height,
@@ -166,17 +149,18 @@ PyImagingPhotoPut(ClientData clientdata, Tcl_Interp* interp,
     return TCL_OK;
 }
 
-/*  Warning -- this does not work at all */
 static int
 PyImagingPhotoGet(ClientData clientdata, Tcl_Interp* interp,
                int argc, const char **argv)
 {
+    Imaging im;
     Tk_PhotoHandle photo;
     Tk_PhotoImageBlock block;
+    int x, y, z;
 
-    if (argc != 2) {
+    if (argc != 3) {
         TCL_APPEND_RESULT(interp, "usage: ", argv[0],
-                         " srcPhoto", (char *) NULL);
+                         " srcPhoto destImage", (char *) NULL);
         return TCL_ERROR;
     }
 
@@ -189,21 +173,26 @@ PyImagingPhotoGet(ClientData clientdata, Tcl_Interp* interp,
         return TCL_ERROR;
     }
 
+    /* get PIL Image handle */
+    im = ImagingFind(argv[2]);
+    if (!im) {
+        TCL_APPEND_RESULT(interp, "bad name", (char*) NULL);
+        return TCL_ERROR;
+    }
+
     TK_PHOTO_GET_IMAGE(photo, &block);
 
-    printf("pixelPtr = %p\n", block.pixelPtr);
-    printf("width = %d\n", block.width);
-    printf("height = %d\n", block.height);
-    printf("pitch = %d\n", block.pitch);
-    printf("pixelSize = %d\n", block.pixelSize);
-    printf("offset = %d %d %d %d\n", block.offset[0], block.offset[1],
-           block.offset[2], block.offset[3]);
+    for (y = 0; y < block.height; y++) {
+        UINT8* out = (UINT8*)im->image32[y];
+        for (x = 0; x < block.pitch; x += block.pixelSize) {
+            for (z=0; z < block.pixelSize; z++) {
+                int offset = block.offset[z];
+                out[x + offset] = block.pixelPtr[y * block.pitch + x + offset];
+            }
+        }
+    }
 
-    TCL_APPEND_RESULT(
-        interp, "this function is not yet supported", (char *) NULL
-        );
-
-    return TCL_ERROR;
+    return TCL_OK;
 }
 
 
@@ -236,11 +225,7 @@ TkImaging_Init(Tcl_Interp* interp)
 #include <psapi.h>
 /* Must be linked with 'psapi' library */
 
-#if PY_VERSION_HEX >= 0x03000000
 #define TKINTER_PKG "tkinter"
-#else
-#define TKINTER_PKG "Tkinter"
-#endif
 
 FARPROC _dfunc(HMODULE lib_handle, const char *func_name)
 {
@@ -365,7 +350,6 @@ int load_tkinter_funcs(void)
  */
 
 /* From module __file__ attribute to char *string for dlopen. */
-#if PY_VERSION_HEX >= 0x03000000
 char *fname2char(PyObject *fname)
 {
     PyObject* bytes;
@@ -375,9 +359,6 @@ char *fname2char(PyObject *fname)
     }
     return PyBytes_AsString(bytes);
 }
-#else
-#define fname2char(s) (PyString_AsString(s))
-#endif
 
 #include <dlfcn.h>
 
@@ -442,6 +423,7 @@ int load_tkinter_funcs(void)
     /* Try loading from the main program namespace first */
     main_program = dlopen(NULL, RTLD_LAZY);
     if (_func_loader(main_program) == 0) {
+        dlclose(main_program);
         return 0;
     }
     /* Clear exception triggered when we didn't find symbols above */
@@ -470,6 +452,7 @@ int load_tkinter_funcs(void)
     /* dlclose probably safe because tkinter has been imported. */
     dlclose(tkinter_lib);
 exit:
+    dlclose(main_program);
     Py_XDECREF(pModule);
     Py_XDECREF(pString);
     return ret;
