@@ -25,13 +25,13 @@ kevin@cazabon.com\n\
 http://www.cazabon.com\n\
 "
 
+#define PY_SSIZE_T_CLEAN
 #include "Python.h" // Include before wchar.h so _GNU_SOURCE is set
 #include "wchar.h"
 #include "datetime.h"
 
 #include "lcms2.h"
 #include "Imaging.h"
-#include "py3.h"
 
 #define PYCMSVERSION "1.0.0 pil"
 
@@ -120,14 +120,9 @@ cms_profile_fromstring(PyObject* self, PyObject* args)
     cmsHPROFILE hProfile;
 
     char* pProfile;
-    int nProfile;
-#if PY_VERSION_HEX >= 0x03000000
+    Py_ssize_t nProfile;
     if (!PyArg_ParseTuple(args, "y#:profile_frombytes", &pProfile, &nProfile))
         return NULL;
-#else
-    if (!PyArg_ParseTuple(args, "s#:profile_fromstring", &pProfile, &nProfile))
-        return NULL;
-#endif
 
     hProfile = cmsOpenProfileFromMem(pProfile, nProfile);
     if (!hProfile) {
@@ -171,11 +166,7 @@ cms_profile_tobytes(PyObject* self, PyObject* args)
         return NULL;
     }
 
-#if PY_VERSION_HEX >= 0x03000000
     ret = PyBytes_FromStringAndSize(pProfile, (Py_ssize_t)nProfile);
-#else
-    ret = PyString_FromStringAndSize(pProfile, (Py_ssize_t)nProfile);
-#endif
 
     free(pProfile);
     return ret;
@@ -591,7 +582,7 @@ cms_profile_is_intent_supported(CmsProfileObject *self, PyObject *args)
 
     /* printf("cmsIsIntentSupported(%p, %d, %d) => %d\n", self->profile, intent, direction, result); */
 
-    return PyInt_FromLong(result != 0);
+    return PyLong_FromLong(result != 0);
 }
 
 #ifdef _WIN32
@@ -690,11 +681,7 @@ _profile_read_int_as_string(cmsUInt32Number nr)
     buf[3] = (char) (nr & 0xff);
     buf[4] = 0;
 
-#if PY_VERSION_HEX >= 0x03000000
     ret = PyUnicode_DecodeASCII(buf, 4, NULL);
-#else
-    ret = PyString_FromStringAndSize(buf, 4);
-#endif
     return ret;
 }
 
@@ -735,12 +722,12 @@ _xyz3_py(cmsCIEXYZ* XYZ)
     cmsXYZ2xyY(&xyY[2], &XYZ[2]);
 
     return Py_BuildValue("(((d,d,d),(d,d,d),(d,d,d)),((d,d,d),(d,d,d),(d,d,d)))",
-			 XYZ[0].X, XYZ[0].Y, XYZ[0].Z,
-			 XYZ[1].X, XYZ[1].Y, XYZ[1].Z,
-			 XYZ[2].X, XYZ[2].Y, XYZ[2].Z,
-			 xyY[0].x, xyY[0].y, xyY[0].Y,
-			 xyY[1].x, xyY[1].y, xyY[1].Y,
-			 xyY[2].x, xyY[2].y, xyY[2].Y);
+        XYZ[0].X, XYZ[0].Y, XYZ[0].Z,
+        XYZ[1].X, XYZ[1].Y, XYZ[1].Z,
+        XYZ[2].X, XYZ[2].Y, XYZ[2].Z,
+        xyY[0].x, xyY[0].y, xyY[0].Y,
+        xyY[1].x, xyY[1].y, xyY[1].Y,
+        xyY[2].x, xyY[2].y, xyY[2].Y);
 }
 
 static PyObject*
@@ -783,9 +770,9 @@ _profile_read_ciexyy_triple(CmsProfileObject* self, cmsTagSignature info)
     /* Note: lcms does all the heavy lifting and error checking (nr of
        channels == 3).  */
     return Py_BuildValue("((d,d,d),(d,d,d),(d,d,d)),",
-			 triple->Red.x, triple->Red.y, triple->Red.Y,
-			 triple->Green.x, triple->Green.y, triple->Green.Y,
-			 triple->Blue.x, triple->Blue.y, triple->Blue.Y);
+        triple->Red.x, triple->Red.y, triple->Red.Y,
+        triple->Green.x, triple->Green.y, triple->Green.Y,
+        triple->Blue.x, triple->Blue.y, triple->Blue.Y);
 }
 
 static PyObject*
@@ -817,12 +804,12 @@ _profile_read_named_color_list(CmsProfileObject* self, cmsTagSignature info)
     for (i = 0; i < n; i++) {
         PyObject* str;
         cmsNamedColorInfo(ncl, i, name, NULL, NULL, NULL, NULL);
-	str = PyUnicode_FromString(name);
-	if (str == NULL) {
- 	    Py_DECREF(result);
-	    Py_INCREF(Py_None);
-	    return Py_None;
-	}
+        str = PyUnicode_FromString(name);
+        if (str == NULL) {
+            Py_DECREF(result);
+            Py_INCREF(Py_None);
+            return Py_None;
+        }
         PyList_SET_ITEM(result, i, str);
     }
 
@@ -844,9 +831,9 @@ static cmsBool _calculate_rgb_primaries(CmsProfileObject* self, cmsCIEXYZTRIPLE*
 
     // transform from our profile to XYZ using doubles for highest precision
     hTransform = cmsCreateTransform(self->profile, TYPE_RGB_DBL,
-				    hXYZ, TYPE_XYZ_DBL,
-				    INTENT_RELATIVE_COLORIMETRIC,
-				    cmsFLAGS_NOCACHE | cmsFLAGS_NOOPTIMIZE);
+                                    hXYZ, TYPE_XYZ_DBL,
+                                    INTENT_RELATIVE_COLORIMETRIC,
+                                    cmsFLAGS_NOCACHE | cmsFLAGS_NOOPTIMIZE);
     cmsCloseProfile(hXYZ);
     if (hTransform == NULL)
         return 0;
@@ -885,31 +872,31 @@ _is_intent_supported(CmsProfileObject* self, int clut)
 
 
     n = cmsGetSupportedIntents(INTENTS,
-			       intent_ids,
-			       intent_descs);
+                               intent_ids,
+                               intent_descs);
     for (i = 0; i < n; i++) {
         int intent = (int) intent_ids[i];
         PyObject* id;
-	PyObject* entry;
+        PyObject* entry;
 
-	/* Only valid for ICC Intents (otherwise we read invalid memory in lcms cmsio1.c). */
-	if (!(intent == INTENT_PERCEPTUAL || intent == INTENT_RELATIVE_COLORIMETRIC
-	      || intent == INTENT_SATURATION || intent == INTENT_ABSOLUTE_COLORIMETRIC))
-	  continue;
+        /* Only valid for ICC Intents (otherwise we read invalid memory in lcms cmsio1.c). */
+        if (!(intent == INTENT_PERCEPTUAL || intent == INTENT_RELATIVE_COLORIMETRIC
+            || intent == INTENT_SATURATION || intent == INTENT_ABSOLUTE_COLORIMETRIC))
+            continue;
 
-	id = PyInt_FromLong((long) intent);
-	entry = Py_BuildValue("(OOO)",
-			      _check_intent(clut, self->profile, intent, LCMS_USED_AS_INPUT) ? Py_True : Py_False,
-			      _check_intent(clut, self->profile, intent, LCMS_USED_AS_OUTPUT) ? Py_True : Py_False,
-			      _check_intent(clut, self->profile, intent, LCMS_USED_AS_PROOF) ? Py_True : Py_False);
-	if (id == NULL || entry == NULL) {
-	    Py_XDECREF(id);
-	    Py_XDECREF(entry);
-	    Py_XDECREF(result);
-	    Py_INCREF(Py_None);
-	    return Py_None;
-	}
-	PyDict_SetItem(result, id, entry);
+        id = PyLong_FromLong((long) intent);
+        entry = Py_BuildValue("(OOO)",
+            _check_intent(clut, self->profile, intent, LCMS_USED_AS_INPUT) ? Py_True : Py_False,
+            _check_intent(clut, self->profile, intent, LCMS_USED_AS_OUTPUT) ? Py_True : Py_False,
+            _check_intent(clut, self->profile, intent, LCMS_USED_AS_PROOF) ? Py_True : Py_False);
+        if (id == NULL || entry == NULL) {
+            Py_XDECREF(id);
+            Py_XDECREF(entry);
+            Py_XDECREF(result);
+            Py_INCREF(Py_None);
+            return Py_None;
+        }
+        PyDict_SetItem(result, id, entry);
     }
     return result;
 }
@@ -966,6 +953,8 @@ _profile_getattr(CmsProfileObject* self, cmsInfoType field)
 static PyObject*
 cms_profile_getattr_product_desc(CmsProfileObject* self, void* closure)
 {
+    PyErr_WarnEx(PyExc_DeprecationWarning,
+                 "product_desc is deprecated. Use Unicode profile_description instead.", 1);
     // description was Description != 'Copyright' || or  "%s - %s" (manufacturer, model) in 1.x
     return _profile_getattr(self, cmsInfoDescription);
 }
@@ -975,42 +964,54 @@ cms_profile_getattr_product_desc(CmsProfileObject* self, void* closure)
 static PyObject*
 cms_profile_getattr_product_description(CmsProfileObject* self, void* closure)
 {
+    PyErr_WarnEx(PyExc_DeprecationWarning,
+                 "product_description is deprecated. Use Unicode profile_description instead.", 1);
     return _profile_getattr(self, cmsInfoDescription);
 }
 
 static PyObject*
 cms_profile_getattr_product_model(CmsProfileObject* self, void* closure)
 {
+    PyErr_WarnEx(PyExc_DeprecationWarning,
+                 "product_model is deprecated. Use Unicode model instead.", 1);
     return _profile_getattr(self, cmsInfoModel);
 }
 
 static PyObject*
 cms_profile_getattr_product_manufacturer(CmsProfileObject* self, void* closure)
 {
+    PyErr_WarnEx(PyExc_DeprecationWarning,
+                 "product_manufacturer is deprecated. Use Unicode manufacturer instead.", 1);
     return _profile_getattr(self, cmsInfoManufacturer);
 }
 
 static PyObject*
 cms_profile_getattr_product_copyright(CmsProfileObject* self, void* closure)
 {
+    PyErr_WarnEx(PyExc_DeprecationWarning,
+                 "product_copyright is deprecated. Use Unicode copyright instead.", 1);
     return _profile_getattr(self, cmsInfoCopyright);
 }
 
 static PyObject*
 cms_profile_getattr_rendering_intent(CmsProfileObject* self, void* closure)
 {
-    return PyInt_FromLong(cmsGetHeaderRenderingIntent(self->profile));
+    return PyLong_FromLong(cmsGetHeaderRenderingIntent(self->profile));
 }
 
 static PyObject*
 cms_profile_getattr_pcs(CmsProfileObject* self, void* closure)
 {
+    PyErr_WarnEx(PyExc_DeprecationWarning,
+                 "pcs is deprecated. Use padded connection_space instead.", 1);
     return PyUnicode_DecodeFSDefault(findICmode(cmsGetPCS(self->profile)));
 }
 
 static PyObject*
 cms_profile_getattr_color_space(CmsProfileObject* self, void* closure)
 {
+    PyErr_WarnEx(PyExc_DeprecationWarning,
+                 "color_space is deprecated. Use padded xcolor_space instead.", 1);
     return PyUnicode_DecodeFSDefault(findICmode(cmsGetColorSpace(self->profile)));
 }
 
@@ -1070,7 +1071,7 @@ cms_profile_getattr_creation_date(CmsProfileObject* self, void* closure)
     }
 
     return PyDateTime_FromDateAndTime(1900 + ct.tm_year, ct.tm_mon, ct.tm_mday,
-				      ct.tm_hour, ct.tm_min, ct.tm_sec, 0);
+                                      ct.tm_hour, ct.tm_min, ct.tm_sec, 0);
 }
 
 static PyObject*
@@ -1083,7 +1084,7 @@ cms_profile_getattr_version(CmsProfileObject* self, void* closure)
 static PyObject*
 cms_profile_getattr_icc_version(CmsProfileObject* self, void* closure)
 {
-    return PyInt_FromLong((long) cmsGetEncodedICCversion(self->profile));
+    return PyLong_FromLong((long) cmsGetEncodedICCversion(self->profile));
 }
 
 static PyObject*
@@ -1100,7 +1101,7 @@ static PyObject*
 cms_profile_getattr_header_flags(CmsProfileObject* self, void* closure)
 {
     cmsUInt32Number flags = cmsGetHeaderFlags(self->profile);
-    return PyInt_FromLong(flags);
+    return PyLong_FromLong(flags);
 }
 
 static PyObject*
@@ -1295,7 +1296,7 @@ cms_profile_getattr_green_primary(CmsProfileObject* self, void* closure)
         result = _calculate_rgb_primaries(self, &primaries);
     if (! result) {
         Py_INCREF(Py_None);
-	return Py_None;
+        return Py_None;
     }
 
     return _xyz_py(&primaries.Green);
@@ -1311,7 +1312,7 @@ cms_profile_getattr_blue_primary(CmsProfileObject* self, void* closure)
         result = _calculate_rgb_primaries(self, &primaries);
     if (! result) {
         Py_INCREF(Py_None);
-	return Py_None;
+        return Py_None;
     }
 
     return _xyz_py(&primaries.Blue);
@@ -1394,11 +1395,11 @@ cms_profile_getattr_icc_measurement_condition (CmsProfileObject* self, void* clo
         geo = "unknown";
 
     return Py_BuildValue("{s:i,s:(ddd),s:s,s:d,s:s}",
-			 "observer", mc->Observer,
-			 "backing", mc->Backing.X, mc->Backing.Y, mc->Backing.Z,
-			 "geo", geo,
-			 "flare", mc->Flare,
-			 "illuminant_type", _illu_map(mc->IlluminantType));
+                         "observer", mc->Observer,
+                         "backing", mc->Backing.X, mc->Backing.Y, mc->Backing.Z,
+                         "geo", geo,
+                         "flare", mc->Flare,
+                         "illuminant_type", _illu_map(mc->IlluminantType));
 }
 
 static PyObject*
@@ -1419,9 +1420,9 @@ cms_profile_getattr_icc_viewing_condition (CmsProfileObject* self, void* closure
     }
 
     return Py_BuildValue("{s:(ddd),s:(ddd),s:s}",
-			 "illuminant", vc->IlluminantXYZ.X, vc->IlluminantXYZ.Y, vc->IlluminantXYZ.Z,
-			 "surround", vc->SurroundXYZ.X, vc->SurroundXYZ.Y, vc->SurroundXYZ.Z,
-			 "illuminant_type", _illu_map(vc->IlluminantType));
+        "illuminant", vc->IlluminantXYZ.X, vc->IlluminantXYZ.Y, vc->IlluminantXYZ.Z,
+        "surround", vc->SurroundXYZ.X, vc->SurroundXYZ.Y, vc->SurroundXYZ.Z,
+        "illuminant_type", _illu_map(vc->IlluminantType));
 }
 
 
@@ -1596,7 +1597,6 @@ setup_module(PyObject* m) {
     return 0;
 }
 
-#if PY_VERSION_HEX >= 0x03000000
 PyMODINIT_FUNC
 PyInit__imagingcms(void) {
     PyObject* m;
@@ -1618,12 +1618,3 @@ PyInit__imagingcms(void) {
 
     return m;
 }
-#else
-PyMODINIT_FUNC
-init_imagingcms(void)
-{
-    PyObject *m = Py_InitModule("_imagingcms", pyCMSdll_methods);
-    setup_module(m);
-    PyDateTime_IMPORT;
-}
-#endif
